@@ -131,6 +131,9 @@ static void tree_select_row_cb (GtkCTree*, GtkCTreeNode*, int, gpointer);
 PanCallback * articlelist_thread_changed         = NULL;
 PanCallback * articlelist_selection_changed      = NULL;
 
+static GString * cur_message_id                  = NULL;
+static GString * prev_message_id                  = NULL;
+
 static GtkStyle * normal_style[2]                = { NULL, NULL };
 static GtkStyle * read_style[2]                  = { NULL, NULL };
 static GtkStyle * watched_style[2]               = { NULL, NULL };
@@ -624,6 +627,19 @@ find_next_article (GtkCTreeNode * node)
 	return retval;
 }
 
+static void
+read_article (Article * a)
+{
+	if (a != NULL)
+	{
+		g_return_if_fail (article_is_valid(a));
+
+		g_string_assign (prev_message_id, cur_message_id->str);
+		g_string_assign (cur_message_id, article_get_message_id(a));
+		text_set_from_article (a);
+	}
+}
+
 static void
 select_and_maybe_read_nolock (GtkCTreeNode * node, gboolean do_read)
 {
@@ -633,11 +649,7 @@ select_and_maybe_read_nolock (GtkCTreeNode * node, gboolean do_read)
 	{
 		articlelist_set_selected_nodes_nolock (&node, 1);
 		if (do_read)
-		{
-			Article * a = articlelist_get_article_from_node (node);
-			if (a != NULL)
-				text_set_from_article (a);
-		}
+			read_article (articlelist_get_article_from_node (node));
 	}
 }
 
@@ -649,14 +661,9 @@ articlelist_read_selected (void)
 	if ((node = articlelist_get_selected_node ()) == NULL)
 		node = gtk_ctree_node_nth (GTK_CTREE(Pan.article_ctree), 1);
 
-	if (node != NULL)	
-	{
-		Article * a = NULL;
-
+	if (node != NULL) {
 		articlelist_set_selected_nodes_nolock (&node, 1);
-		a =  articlelist_get_article_from_node (node);
-		if (a != NULL)
-			text_set_from_article (a);
+		read_article (articlelist_get_article_from_node (node));
 	}
 }
 
@@ -776,6 +783,19 @@ articlelist_read_next_unread_thread (void)
 	select_and_maybe_read_nolock (node, navigate_read_on_select);
 }
 
+gboolean
+articlelist_has_prev_read (void)
+{
+	return prev_message_id->len != 0;
+}
+
+void
+articlelist_read_last_read (void)
+{
+	if (articlelist_has_prev_read())
+		articlelist_read_message_id (prev_message_id->str);
+}
+
 void
 articlelist_read_prev (void)
 {
@@ -2511,7 +2531,7 @@ articlelist_selection_changed_cb (gpointer call_object,
 
 	/* decide whether or not to load it */
         if (do_load && single_article!=NULL && article_is_valid(single_article))
-		text_set_from_article (single_article);
+		read_article (single_article);
 
 	/* reset everything for the next time around */
 	button_click_count = -1;
@@ -2821,6 +2841,9 @@ create_articlelist_ctree (void)
 	GtkWidget * vbox;
 	const char * titles[COLUMNS];
 
+	cur_message_id = g_string_new (NULL);
+	prev_message_id = g_string_new (NULL);
+
 	my_articles = g_ptr_array_new ();
 
 	/* callbacks */
@@ -3441,6 +3464,8 @@ clear_group_nolock (void)
 		group_unref_articles (my_group, NULL);
 		g_ptr_array_set_size (my_articles, 0);
 		my_group = NULL;
+		g_string_truncate (cur_message_id, 0);
+		g_string_truncate (prev_message_id, 0);
 		gui_queue_add (fire_group_changed, my_group);
 	}
 }
@@ -3475,6 +3500,8 @@ set_group_mainthread_end (gpointer p)
 	/* update the model */
 	old_group = my_group;
 	my_group = group;
+	g_string_truncate (cur_message_id, 0);
+	g_string_truncate (prev_message_id, 0);
 	pan_g_ptr_array_assign (my_articles, articles->pdata, articles->len);
 	if (old_group != NULL) {
 		file_headers_save (old_group, NULL);
@@ -3494,9 +3521,13 @@ set_group_mainthread_end (gpointer p)
 
 	/* Now that the articlelist is fully loaded, */
 	/* get new headers, if required.             */
-	if (fetch_new_on_group_load)
-		queue_add (TASK(task_headers_new (group, HEADERS_NEW)));
-
+	if (fetch_new_on_group_load) {
+		if(download_bodies_too)
+			queue_add (TASK(task_headers_new_with_bodies (group, HEADERS_NEW)));
+		else
+			queue_add (TASK(task_headers_new (group, HEADERS_NEW)));
+	}
+	
 	debug_exit ("set_group_mainthread_end");
 	return 0;
 }
@@ -3601,7 +3632,10 @@ articlelist_set_group_nolock (Group * group)
 
 		clear_group_nolock ();
 
-		task = TASK (task_headers_new (group, HEADERS_NEW));
+		if(download_bodies_too)
+			task = TASK (task_headers_new_with_bodies (group, HEADERS_NEW));
+		else
+			task = TASK (task_headers_new (group, HEADERS_NEW));
 		pan_callback_add (task->task_ran_callback, headers_fetched, group);
 		queue_add (task);
 	}
