@@ -33,6 +33,8 @@
 #include <dirent.h>
 #include <pthread.h>
 
+#include <gmime/memchunk.h>
+
 #include <pan/base/acache.h>
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
@@ -44,9 +46,9 @@
 PanCallback* acache_bodies_added_callback = NULL;
 PanCallback* acache_bodies_removed_callback = NULL;
 
-static gint acache_size = 0;
+static size_t acache_size = 0;
 static pthread_mutex_t entries_mutex = PTHREAD_MUTEX_INITIALIZER;
-static GMemChunk * acache_entry_chunk = NULL;
+static MemChunk * acache_entry_chunk = NULL;
 static GHashTable * messageid_to_entry = NULL;
 
 typedef struct
@@ -78,10 +80,10 @@ get_file_message_id (const gchar * filename, gchar * msgid, gint msgid_len)
 	fp = fopen (filename, "r");
 	if (fp != NULL) {
 		const gchar * key = "Message-I";
-		const size_t key_len = strlen (key);
+		const size_t key_len = 9;
 		gboolean found;
 		for (found=FALSE; !found && fgets(buf,sizeof(buf),fp); ) {
-			if (strlen(buf) < 3)
+			if (buf[0]=='\0' || buf[1]=='\0') /* watch for header/body delimiter */
 				break;
 			if (!memcmp (key, buf, key_len))
 				found = TRUE;
@@ -121,9 +123,10 @@ get_file_message_id (const gchar * filename, gchar * msgid, gint msgid_len)
 static gchar*
 acache_get_path (void)
 {
-	return g_strdup_printf ("%s%ccache",
-	                        data_dir,
-	                        G_DIR_SEPARATOR);
+	gchar * pch = g_strdup_printf ("%s%ccache", data_dir, G_DIR_SEPARATOR);
+	pan_normalize_filename_inplace (pch);
+	return pch;
+
 }
 
 static gchar*
@@ -163,6 +166,11 @@ void
 acache_init (void)
 {
 	gchar * pch;
+	struct dirent * dirent_p;
+	gchar * dirpath;
+	DIR * dir_p;
+	gchar message_id[1024];
+	gchar path [PATH_MAX];
 	debug_enter ("acache_init");
 
 	/* we don't need to call this multiple times */
@@ -179,50 +187,50 @@ acache_init (void)
 	acache_bodies_removed_callback = pan_callback_new ();
 
 	/* init the message id hash  */
-	acache_entry_chunk = g_mem_chunk_create (AcacheEntry, 256, G_ALLOC_AND_FREE);
+	acache_entry_chunk = memchunk_new (sizeof(AcacheEntry), 256, FALSE);
 	messageid_to_entry = g_hash_table_new (g_str_hash, g_str_equal);
 	acache_size = 0;
-	if (1)
+	dirpath = acache_get_path ();
+	dir_p = opendir (dirpath);
+	if (dir_p == NULL) {
+		if (errno)
+			log_add_va (LOG_ERROR, _("Error reading \"%s\": %s."), dirpath, g_strerror(errno));
+		else
+			log_add_va (LOG_ERROR, _("Error reading \"%s\"."), dirpath);
+	}
+
+	while (dir_p!=NULL && (dirent_p = readdir(dir_p)))
 	{
-		struct dirent * dirent_p = NULL;
-		gchar * dirpath = acache_get_path ();
-		DIR * dir_p = opendir (dirpath);
-		gchar message_id[1024];
-		gchar path [PATH_MAX];
+		struct stat stat_p;
 
-		while (dir_p!=NULL && (dirent_p = readdir(dir_p)))
+		if (!string_ends_with(dirent_p->d_name, ".msg"))
+			continue;
+
+		g_snprintf (path, sizeof(path), "%s%c%s", dirpath, G_DIR_SEPARATOR, dirent_p->d_name);
+		if (stat(path, &stat_p) == 0)
 		{
-			struct stat stat_p;
-
-			if (!string_ends_with(dirent_p->d_name, ".msg"))
-				continue;
-
-			sprintf (path, "%s%c%s", dirpath, G_DIR_SEPARATOR, dirent_p->d_name);
-			if (stat(path, &stat_p) == 0)
+			if (get_file_message_id (path, message_id, sizeof(message_id)))
 			{
-				if (get_file_message_id (path, message_id, sizeof(message_id)))
-				{
-					AcacheEntry * entry = g_chunk_new (AcacheEntry, acache_entry_chunk);
-					entry->message_id = g_strdup (message_id);
-					entry->size = stat_p.st_size;
-					entry->date = stat_p.st_mtime;
-					entry->refcount = 0;
-					g_hash_table_insert (messageid_to_entry, entry->message_id, entry);
-					acache_size += entry->size;
-				}
+				AcacheEntry * entry = (AcacheEntry*) memchunk_alloc (acache_entry_chunk);
+				entry->message_id = g_strdup (message_id);
+				entry->size = stat_p.st_size;
+				entry->date = stat_p.st_mtime;
+				entry->refcount = 0;
+				g_hash_table_insert (messageid_to_entry, entry->message_id, entry);
+				acache_size += entry->size;
 			}
 		}
-
-		if (dir_p!=NULL)
-			closedir (dir_p);
-
-		g_free (dirpath);
-
-		log_add_va (LOG_INFO, _("Article cache contains %.1f MB in %d files"),
-			(double)acache_size/(1024*1024),
-			(int)g_hash_table_size(messageid_to_entry));
 	}
 
+	if (dir_p!=NULL)
+		closedir (dir_p);
+
+	g_free (dirpath);
+
+	log_add_va (LOG_INFO, _("Article cache contains %.1f MB in %d files"),
+		(double)acache_size/(1024*1024),
+		(int)g_hash_table_size(messageid_to_entry));
+
 	debug_exit ("acache_init");
 }
 
@@ -242,7 +250,7 @@ acache_shutdown (void)
 	g_hash_table_destroy (messageid_to_entry);
 	messageid_to_entry = NULL;
 
-	g_mem_chunk_destroy (acache_entry_chunk);
+	memchunk_destroy (acache_entry_chunk);
 	acache_entry_chunk = NULL;
 }
 
@@ -252,6 +260,7 @@ acache_shutdown (void)
 ****
 ***/
 
+/* this must be called inside an entries_mutex lock */
 static void
 acache_file_update_refcount_nolock (const gchar ** message_ids,
                                     gint           message_id_qty,
@@ -270,7 +279,7 @@ acache_file_update_refcount_nolock (const gchar ** message_ids,
 		entry = g_hash_table_lookup (messageid_to_entry, message_id);
 		if (entry == NULL)
 		{
-			entry = g_chunk_new (AcacheEntry, acache_entry_chunk);
+			entry = (AcacheEntry*) memchunk_alloc (acache_entry_chunk);
 			entry->refcount = 0;
 			entry->message_id = g_strdup (message_id);
 			entry->size = 0;
@@ -312,8 +321,7 @@ acache_file_checkin (const gchar ** message_ids,
 	acache_file_update_refcount_nolock (message_ids, message_id_qty, -1);
 	pthread_mutex_unlock (&entries_mutex);
 
-	if (acache_size > (acache_max_megs*1024*1024))
-		acache_expire ();
+	acache_expire ();
 }
 
 /***
@@ -325,6 +333,7 @@ acache_file_checkin (const gchar ** message_ids,
 void
 acache_set_message (const gchar    * message_id,
                     const gchar    * message,
+                    guint            message_len,
                     gboolean         checkout)
 {
        	FILE * fp = NULL;
@@ -341,25 +350,35 @@ acache_set_message (const gchar    * message_id,
 	if (fp != NULL)
 	{
 		/* write the message */
-		fwrite (message, sizeof(char), strlen(message), fp);
+		const size_t bytes_written = fwrite (message, sizeof(char), message_len, fp);
 		fclose (fp);
 
-		/* update our tables */
-		pthread_mutex_lock (&entries_mutex);
-		acache_file_update_refcount_nolock (&message_id, 1, checkout?1:0);
-		entry = g_hash_table_lookup (messageid_to_entry, message_id);
-		g_assert (entry!=NULL);
-		entry->size = strlen (message);
-		entry->date = time (NULL);
-		pthread_mutex_unlock (&entries_mutex);
+		if (bytes_written < message_len)
+		{
+			/* couldn't save the whole message */
+			gchar * path = acache_get_path ();
+			log_add_va (LOG_ERROR, _("Error saving article \"%s\" (is %s full?)"), message_id, path);
+			g_free (path);
+		}
+		else
+		{
+			/* update our tables */
+			pthread_mutex_lock (&entries_mutex);
+			acache_file_update_refcount_nolock (&message_id, 1, checkout?1:0);
+			entry = g_hash_table_lookup (messageid_to_entry, message_id);
+			g_assert (entry!=NULL);
+			entry->size = message_len;
+			entry->date = time (NULL);
+			pthread_mutex_unlock (&entries_mutex);
 
-		/* notify everyone that this message has been added */
-		pan_callback_call (acache_bodies_added_callback, (gpointer)message_id, NULL);
+			/* notify everyone that this message has been added */
+			pan_callback_call (acache_bodies_added_callback, (gpointer)message_id, NULL);
 
-		/* if the acache is too big, purge the old stuff */
-		acache_size += entry->size;
-		if (acache_size > (acache_max_megs*1024*1024))
+			/* if the acache is too big, purge the old stuff */
+			acache_size += entry->size;
 			acache_expire ();
+			
+		}
 	}
 }
 
@@ -375,7 +394,7 @@ acache_remove_entry (AcacheEntry * entry)
 	g_return_if_fail (entry!=NULL);
 
 	/* let everyone know */
-	debug2 (DEBUG_ACACHE, "Expiring %s with refcount %d", entry->message_id, entry->refcount);
+	debug2 (DEBUG_ACACHE, "Removing %s from cache with refcount %d", entry->message_id, entry->refcount);
 	pan_callback_call (acache_bodies_removed_callback, (gpointer)entry->message_id, NULL);
 
 	/* update the acache size */
@@ -388,22 +407,28 @@ acache_remove_entry (AcacheEntry * entry)
 	/* update acache tables */
 	g_hash_table_remove (messageid_to_entry, entry->message_id);
 	g_free (entry->message_id);
-	g_mem_chunk_free (acache_entry_chunk, entry);
+	memchunk_free (acache_entry_chunk, entry);
 }
 
 gchar*
 acache_get_message (const gchar * message_id)
 {
 	gchar * buf = NULL;
-	gchar filename[PATH_MAX];
 
-	if (acache_get_filename (message_id, filename, PATH_MAX))
+	/* sanity clause */
+	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
+
+	if (acache_has_message (message_id))
 	{
-		GArray* file = read_file (filename);
-		if (file != NULL)
+		gchar filename[PATH_MAX];
+		if (acache_get_filename (message_id, filename, PATH_MAX))
 		{
-			buf = file->data;
-			g_array_free (file, FALSE);
+			GArray* file = read_file (filename);
+			if (file != NULL)
+			{
+				buf = file->data;
+				g_array_free (file, FALSE);
+			}
 		}
 	}
 
@@ -418,6 +443,8 @@ acache_has_message (const gchar * message_id)
 	gboolean retval = FALSE;
 	AcacheEntry * entry = NULL;
 
+	g_return_val_if_fail (is_nonempty_string(message_id), FALSE);
+
 	entry = g_hash_table_lookup (messageid_to_entry, message_id);
 	if (entry != NULL)
 		retval = entry->size != 0;
@@ -449,6 +476,8 @@ acache_expire_messages (const gchar   ** message_ids,
 	g_return_if_fail (message_ids!=NULL);
 	g_return_if_fail (message_id_qty>0);
 
+	pthread_mutex_lock (&entries_mutex);
+
 	for (i=0; i<message_id_qty; ++i)
 	{
 		AcacheEntry * entry = NULL;
@@ -462,6 +491,10 @@ acache_expire_messages (const gchar   ** message_ids,
 		if (entry != NULL)
 			acache_remove_entry (entry);
 	}
+
+	memchunk_clean (acache_entry_chunk);
+
+	pthread_mutex_unlock (&entries_mutex);
 }
 
 
@@ -469,13 +502,13 @@ static gint
 acache_expire_to_size (gdouble max_megs)
 {
 	gint files_removed = 0;
-	const glong cache_max = (glong)(max_megs * 1024 * 1024);
+	const size_t cache_max = (size_t)(max_megs * 1024 * 1024);
 	debug_enter ("acache_expire_to_size");
 
 	/* lock the entries mutex so that our array doesn't wind up holding
 	   pointers to free'd entries */
 	pthread_mutex_lock (&entries_mutex);
-
+	debug2 (DEBUG_ACACHE, "expiring to %lu; current size is %lu", cache_max, acache_size);
 	if (cache_max < acache_size)
 	{
 		gint i;
@@ -498,6 +531,7 @@ acache_expire_to_size (gdouble max_megs)
 
 		/*  cleanup */
 		g_ptr_array_free (entries, TRUE);
+		memchunk_clean (acache_entry_chunk);
 	}
 
 	/* log */
