@@ -81,8 +81,7 @@ normalize_subject (gchar * buf, const Article * a)
 	{
 		/* strip multipart information */
 		if (multipart && (*in=='('||*in=='[') && isdigit((guchar)in[1])) {
-			const char ch = *in=='(' ? ')' : ']';
-			while (*in && *in!=ch)
+			while (*in && *in!=']' && *in!=')')
 				++in;
 			continue;
 		}
@@ -142,31 +141,33 @@ normalize_articles (Article    ** articles,
 	}
 }
 
+
 static int
-compare_pA_to_pA_by_subject (const void * va, const void * vb)
+compare_pA_to_pA_by_part (const void * va, const void * vb)
 {
-	int value;
+	int             value;
 	const Article * a = (const Article *)va;
 	const Article * b = (const Article *)vb;
-	gboolean a_is_reply;
-	gboolean b_is_reply;
+	gboolean        a_is_reply;
+	gboolean        b_is_reply;
 
-	/* subject is the primary key, of course... */
-	if ((value = *a->subject - *b->subject))
-		return value;
+	/* already normalized, so check multipart first */
+	if (a->parts == b->parts)
+		return a->part - b->part;
+
+	/* order by subject */
 	if ((value = strcmp (a->subject, b->subject)))
 		return value;
 
+	/* the rest is probably unnecessary. just carry-over from
+	 * compare_pA_to_pA_by_subject */
+
 	/* if one but not both is a reply, the reply goes second */
 	a_is_reply = skip_reply_leader (a->subject) != a->subject ? 1 : 0;
 	b_is_reply = skip_reply_leader (b->subject) != b->subject ? 1 : 0;
 	if (a_is_reply != b_is_reply)
 		return a_is_reply ? 1 : -1;
 
-	/* check multipart */
-	if ((value = a->part - b->part))
-		return value;
-
 	/* oldest goes first... */
 	if (a->date < b->date)
 		value = -1;
@@ -483,6 +484,7 @@ thread_articles (GPtrArray    * articles)
 	Norm * norm;
 	Norm * sorted_norm;
 	GArray * buf = NULL;
+	GString * tmp_references;
 
 	/* sanity clause */
 	g_return_if_fail (articles!=NULL);
@@ -524,6 +526,7 @@ thread_articles (GPtrArray    * articles)
 
 
 	/* thread the articles */
+       	tmp_references = g_string_new (NULL);
 	for (i=0; i!=qty; ++i)
 	{
 		const gchar * references;
@@ -533,11 +536,12 @@ thread_articles (GPtrArray    * articles)
 
 		/* thread by reference
 		   (except for parts 2...n of multiparts, which need to be threaded by multipart) */
-		references = article_get_header (a, HEADER_REFERENCES);
+		references = a->references;
 		if (a->parts<2 && references!=NULL && *references=='<')
 		{
-			gchar * tmp_references = g_strdup (references);
-			gchar * message_id = strrchr (tmp_references, '<');
+			gchar * message_id;
+			g_string_assign (tmp_references, references);
+		       	message_id = strrchr (tmp_references->str, '<');
 
 			while (parent==NULL && is_nonempty_string(message_id))
 			{
@@ -572,11 +576,9 @@ thread_articles (GPtrArray    * articles)
 
 				/* if we couldn't find the ancestor, march up the References string */
 				*message_id = '\0';
-				g_strchomp (tmp_references);
-				message_id = strrchr (tmp_references, '<');
+				g_strchomp (tmp_references->str);
+				message_id = strrchr (tmp_references->str, '<');
 			}
-
-			g_free (tmp_references);
 		}
 
 
@@ -654,7 +656,7 @@ thread_articles (GPtrArray    * articles)
 	/* right now all the children are normalized; point to articles */
 	for (i=0; i!=qty; ++i) {
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		a->threads = g_slist_sort (a->threads, compare_pA_to_pA_by_subject);
+		a->threads = g_slist_sort (a->threads, compare_pA_to_pA_by_part);
 	}
 
 	/* calculate new/unread child counts */
@@ -686,6 +688,7 @@ thread_articles (GPtrArray    * articles)
 	g_free (norm);
 	g_free (norm_str_buf);
 	g_free (sorted_norm);
+	g_string_free (tmp_references, TRUE);
 	if (buf != NULL)
 		g_array_free (buf, TRUE);
 }
