@@ -702,7 +702,7 @@ article_get_extra_header (const Article * a, const gchar * key)
 	const gchar * retval = NULL;
 
 	/* sanity clause */
-	g_return_val_if_fail (article_is_valid(a), NULL);
+	g_return_val_if_fail (a!=NULL, NULL);
 	g_return_val_if_fail (is_nonempty_string(key), NULL);
 
 	if (a->headers!=NULL)
@@ -1077,17 +1077,17 @@ article_get_headers (const Article * a)
 	return retval;
 }
 
-gchar*
+char*
 article_get_body (const Article * a)
 {
-	gchar * retval = NULL;
+	char * retval = NULL;
 
 	g_return_val_if_fail (article_is_valid(a), g_strdup(""));
 
 	/* see if we've got the body hidden in a header... */
 	if (retval==NULL)
 	{
-		const gchar * body = article_get_extra_header (a, PAN_BODY);
+		const char * body = article_get_extra_header (a, PAN_BODY);
 		if (body != NULL)
 			retval = g_strdup (body);
 	}
@@ -1095,12 +1095,12 @@ article_get_body (const Article * a)
 	/* see if we've got it in acache... */
 	if (retval==NULL)
 	{
-		const gchar * message_id = article_get_message_id (a);
+		const char * message_id = article_get_message_id (a);
 		if (acache_has_message (message_id))
 		{
-			gchar * s = acache_get_message (message_id);
+			char * s = acache_get_message (message_id);
 			gboolean is_html = FALSE;
-			GMimeMessage * m = pan_g_mime_parser_construct_message (s);
+			GMimeMessage * m = pan_g_mime_parser_construct_message ((const char **)&s, 1);
 			retval = g_mime_message_get_body (m, TRUE, &is_html);
 			g_mime_object_unref (GMIME_OBJECT(m));
 			g_free (s);
@@ -1111,14 +1111,14 @@ article_get_body (const Article * a)
 }
 
 gboolean
-article_get_header_run_hash_from_text (const gchar    * article_text,
+article_get_header_run_hash_from_text (const char     * article_text,
                                        GHashTable    ** makeme_hash,
                                        gpointer       * allocme_buf)
 {
 	guint line_qty;
 	guint line_len;
-	const gchar * line; 
-	const gchar * march;
+	const char * line; 
+	const char * march;
 	GHashTable * hash;
 	Run * buf;
 	guint buf_qty;
@@ -1148,7 +1148,7 @@ article_get_header_run_hash_from_text (const gchar    * article_text,
 	while (get_next_token_run (march, '\n', &march, &line, &line_len))
 	{
 		guint key_len = 0;
-		const gchar * key = NULL;
+		const char * key = NULL;
 
 		if (line_len == 0) /* \n\n -- end of headers */
 			break;
@@ -1156,7 +1156,7 @@ article_get_header_run_hash_from_text (const gchar    * article_text,
 		if (get_next_token_run (line, ':', NULL, &key, &key_len))
 		{
 			gint val_len = 0;
-			const gchar * val = NULL;
+			const char * val = NULL;
 			Run * key_run = buf + buf_qty++;
 			Run * val_run = buf + buf_qty++;
 
@@ -1179,14 +1179,14 @@ article_get_header_run_hash_from_text (const gchar    * article_text,
 
 static void
 article_set_author_from_header_impl (Article       * a,
-                                     const gchar   * header_from,
+                                     const char    * header_from,
                                      gboolean        init)
 {
-	gchar addr_buf[512];
-	gchar real_buf[512];
-	gchar * addr = addr_buf;
-	gchar * real = real_buf;
-	gchar * pch;
+	char addr_buf[512];
+	char real_buf[512];
+	char * addr = addr_buf;
+	char * real = real_buf;
+	char * pch;
 
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (a->group!=NULL);
@@ -1249,8 +1249,8 @@ article_set_author_from_header (Article         * a,
 
 
 static void
-set_from_raw_init_header_gmhfunc (const gchar * name,
-                                  const gchar * value,
+set_from_raw_init_header_gmhfunc (const char * name,
+                                  const char * value,
                                   gpointer data)
 {
 	Article * a = ARTICLE(data);
@@ -1264,19 +1264,19 @@ set_from_raw_init_header_gmhfunc (const gchar * name,
 
 void
 article_set_from_raw_message   (Article         * a,
-                                const gchar     * text)
+                                const char      * text)
 {
 	guint16 lines;
 	GMimeMessage * msg;
-	gchar * body;
-	const gchar * pch;
+	char * body;
+	const char * pch;
 	gboolean foo;
 	debug_enter ("article_set_from_raw_message");
 
 	/* get the message */
 	g_return_if_fail (a != NULL);
 	g_return_if_fail (is_nonempty_string(text));
-	msg = pan_g_mime_parser_construct_message (text);
+	msg = pan_g_mime_parser_construct_message ((const char **)&text, 1);
 	g_return_if_fail (msg != NULL);
 
 	/* body */
@@ -1304,7 +1304,7 @@ article_set_from_raw_message   (Article         * a,
 
 	/* fallback if no message-id */
 	if (!is_nonempty_string(a->message_id)) {
-		gchar * id;
+		char * id;
 		g_warning (_("Couldn't parse a Message-ID from the raw message!"));
 		id = gnksa_generate_message_id_from_email_addr (a->author_addr);
 		article_init_header (a, HEADER_MESSAGE_ID, id, DO_CHUNK);
