@@ -49,7 +49,7 @@ group_new (Server * server, const char * name)
 {
 	Group * group;
 	
-	g_return_val_if_fail (server!=NULL, NULL);
+	g_return_val_if_fail (server_is_valid (server), NULL);
 	pan_warn_if_fail (is_nonempty_string(name));
 
 	group = server_alloc_new_group (server);
@@ -360,9 +360,13 @@ group_mark_all_read (Group * g, gboolean read)
 	/* sanity clause */
 	g_return_if_fail (group_is_valid(g));
 
-	/* mark all the articles as read.  use articles_set_read for this because
-	   that updates all the unread_children fields, marks crossposts as read,
-	   etc... */
+	/* ensure the articles are loaded */
+	group_ref_articles (g, NULL);
+
+	/* mark all the articles as read.  use articles_set_read for this 
+	   because that updates all the unread_children fields, marks 
+	   crossposts as read, etc... */
+
 	g_mutex_lock (g->article_mutex);
 	articles = group_get_article_array (g);
 	if (articles->len > 0)
@@ -382,6 +386,9 @@ group_mark_all_read (Group * g, gboolean read)
 	/* update group counts */
 	group_set_article_read_qty (g, (read ? g->article_qty : 0));
 
+	/* unref the articles */
+	group_unref_articles (g, NULL);
+
 	debug_exit ("group_mark_all_read");
 }
 
@@ -405,7 +412,7 @@ gboolean
 group_is_valid (const Group * group)
 {
 	g_return_val_if_fail (group!=NULL, FALSE);
-	g_return_val_if_fail (group->server!=NULL, FALSE);
+	g_return_val_if_fail (server_is_valid (group->server), FALSE);
 	g_return_val_if_fail (group->article_mutex!=NULL, FALSE);
 	g_return_val_if_fail (is_nonempty_string(group->name), FALSE);
 
@@ -465,6 +472,10 @@ group_empty (Group * group, gboolean clear_counts)
 	group_remove_articles (group, a);
 	g_ptr_array_free (a, TRUE);
 
+	/* reset the counters */
+	group_set_article_qty (group, 0);
+	group_set_article_read_qty (group, 0);
+
 	/* if user wants to clear the range, do that too */
 	if (clear_counts)
 		group_set_article_range (group, 0, 0);
@@ -832,7 +843,6 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 		/* rethread the affected articles */
 		if (g_hash_table_size(rethread_hash) > 0) {
 			pan_hash_to_ptr_array (rethread_hash, rethread_a);
-			odebug1 ("rethreading %u articles because relatives were removed", rethread_a->len);
 			thread_articles (rethread_a);
 		}
 
@@ -910,9 +920,34 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 
 static GStaticMutex article_refcount_mutex = G_STATIC_MUTEX_INIT;
 
+static gboolean 
+group_ref_articles_if_loaded (Group * group, StatusItem * status)               
+{                                                                               
+        gboolean retval = FALSE;                                                
+        debug_enter ("group_ref_articles_if_loaded");                           
+
+        /* sanity clause */                                                     
+        g_return_val_if_fail (group_is_valid(group), FALSE);                    
+        g_return_val_if_fail (group->_articles_refcount>=0, FALSE);             
+
+        /* ref if reffed */                                                     
+        g_static_mutex_lock (&article_refcount_mutex);                          
+
+        if (group->_articles_refcount > 1) {                                    
+                ++group->_articles_refcount;                                    
+                retval = TRUE;                                                  
+        }                                                                       
+        g_static_mutex_unlock (&article_refcount_mutex);                        
+
+        debug_exit ("group_ref_articles_if_loaded");                            
+
+        return retval;                                                          
+}                                                                               
+
 void
 group_ref_articles (Group * group, StatusItem * status)
 {
+
 	debug_enter ("group_ref_articles");
 
 	/* sanity clause */
@@ -1101,12 +1136,13 @@ group_remove_crossposts (Group     * g,
 		         gulong    * numbers,
                          gint        number_qty)
 {
+	/* sanity clause */
 	g_return_if_fail (group_is_valid(g));
 	g_return_if_fail (numbers!=NULL);
 	g_return_if_fail (number_qty>0);
 
 	/* remove articles if they're already present */
-	if (g->_articles_refcount>0)
+	if (group_ref_articles_if_loaded (g, NULL))
 	{
 		gint i;
 		GPtrArray * articles;
@@ -1114,6 +1150,7 @@ group_remove_crossposts (Group     * g,
 
 		/* get a list of articles sorted by number for fast lookup */
 		remove = g_ptr_array_new ();
+		pan_g_ptr_array_reserve (remove, number_qty);
 		articles = group_get_article_array (g);
 		qsort (articles->pdata, articles->len, sizeof(Article*), compare_ppA_to_ppA_by_number);
 
@@ -1143,6 +1180,7 @@ group_remove_crossposts (Group     * g,
 
 		g_ptr_array_free (remove, TRUE);
 		g_ptr_array_free (articles, TRUE);
+		group_unref_articles (g, NULL);
 	}
 
 	/* flag articles for deletion in case they get downloaded later */
@@ -1176,7 +1214,8 @@ GHashTable*
 group_get_articles (Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), NULL);
- 
+
+	/* FIXME: shouldn't this load the articles ??? */
 	if (group->_articles == NULL)
 		group->_articles = g_hash_table_new (g_str_hash, g_str_equal);
 
