@@ -480,20 +480,24 @@ filter_xml_to_string  (Filter  ** filters,
 	return retval;
 }
 
-static void
+static gboolean
 filter_xml_fwrite (FILE        * fp,
                    FilterTop  ** filters,
                    gint          filter_qty)
 {
-	gchar * pch;
+	gchar  * pch;
+	size_t   to_write, written;
 
-	g_return_if_fail (fp!=NULL);
-	g_return_if_fail (filters!=NULL);
-	g_return_if_fail (filter_qty>0);
+	g_return_val_if_fail (fp!=NULL, 0);
+	g_return_val_if_fail (filters!=NULL, 0);
+	g_return_val_if_fail (filter_qty>0, 0);
 
 	pch = filter_xml_to_string ((Filter**)filters, filter_qty);
-	fwrite (pch, sizeof(char), strlen(pch), fp);
+	to_write = strlen (pch);
+	written = fwrite (pch, sizeof(char), to_write, fp);
 	g_free (pch);
+
+	return to_write == written;
 }
 
 void
@@ -507,24 +511,34 @@ filter_xml_write (const gchar * filename,
 
 	/* write the file */
 
-	if (filter_qty == 0)
-	{
+	if (filter_qty == 0) {
 		unlink (filename);
 	}
-	else
-	{
-		FILE *fp = fopen (filename, "w+");
+	else {
+		FILE     * fp;
+		gchar    * tmp_path = pan_make_temp (&fp);
+		gboolean   ok = fp ? TRUE : FALSE;
 
-		if (fp == NULL) {
-			log_add_va (LOG_ERROR,
-				_("Can't write filters to `%s': %s"),
-				filename, g_strerror(errno));
-		} else {
-			filter_xml_fwrite (fp, filters, filter_qty);
+		if (ok) {
+			ok = filter_xml_fwrite (fp, filters, filter_qty);
 			fclose (fp);
+		}
+
+		if (ok) {
+			ok = pan_rename (tmp_path, filename);
+		}
+
+		if (ok) {
 			log_add_va (LOG_INFO, _("Wrote filters to `%s'"),
 				filename);
 		}
+	       	else {
+			log_add_va (LOG_ERROR,
+				_("Can't write filters to `%s': %s"),
+				filename, g_strerror(errno));
+		}
+
+		g_free (tmp_path);
 	}
 }
 
