@@ -30,11 +30,11 @@
 #include <pan/base/file-headers.h>
 #include <pan/base/gnksa.h>
 #include <pan/base/log.h>
+#include <pan/base/pan-config.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/serverlist.h>
 #include <pan/base/status-item.h>
 
-#include <pan/filters/filter-ui.h>
 #include <pan/rules/rule-ui.h>
 
 #include <pan/identities/identity.h>
@@ -45,6 +45,7 @@
 #include <pan/article-find.h>
 #include <pan/articlelist.h>
 #include <pan/article-list-toolbar-mediator.h>
+#include <pan/filter-ui.h>
 #include <pan/globals.h>
 #include <pan/group-action.h>
 #include <pan/group-ui.h>
@@ -72,7 +73,8 @@
 GtkTooltips * ttips = NULL;
 GdkColormap * cmap = NULL;
 
-static unsigned int gui_refresh_timeout_id = 0;
+static unsigned int gui_refresh_timer_id = 0;
+static unsigned int gui_queue_timer_id = 0;
 static double KBps = 0.0;
 static GtkWidget * taskbar = NULL;
 static GtkWidget * error_button = NULL;
@@ -91,11 +93,12 @@ static ViewStruct views[VIEW_QTY];
 **/
 
 static void exit_cb (void);
-static gint gui_page_get (void);
-static GtkWidget* gui_page_get_pane (gint page);
+static int gui_page_get_nolock (void);
+static GtkWidget* gui_page_get_pane_nolock (int page);
 static gboolean gui_key_press_cb (GtkWidget*, GdkEventKey*, gpointer);
 static int window_delete_event_cb (GtkWidget*, GdkEvent*, gpointer);
-static int gui_refresh_timeout_cb (gpointer);
+static int gui_refresh_timer_cb (gpointer);
+static int gui_queue_timer_cb (gpointer);
 static void update_menus_cb                      (gpointer, gpointer, gpointer);
 static void log_entry_added_cb                   (gpointer, gpointer, gpointer);
 static void status_item_active_changed_cb        (gpointer, gpointer, gpointer);
@@ -389,6 +392,10 @@ static GnomeUIInfo navigate_menu[] =
 	                              N_("Previous Thread"),
 	                              articlelist_read_prev_thread,
 	                              GNOME_STOCK_MENU_BACK, 'V', GDK_CONTROL_MASK),
+	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("Previous _read article"),	
+	                              N_("Previous read article"),
+	                              articlelist_read_last_read,
+	                              GNOME_STOCK_MENU_BACK, 'V', GDK_SHIFT_MASK),
 	GNOMEUIINFO_ITEM_STOCK_ACCEL (N_("_Parent Article"),
 	                              N_("Parent Article"),
 	                              articlelist_read_parent,
@@ -455,8 +462,8 @@ pan_manual_url (void)
 static void
 send_feedback_cb (GtkWidget * w)
 {
-	gchar * from;
-	gchar * msg_id;
+	char * from;
+	char * msg_id;
 	Group * sendlater;
 	Article * a;
 	Identity * id;
@@ -488,9 +495,7 @@ send_feedback_cb (GtkWidget * w)
 static void
 dialog_about_cb (void)
 {
-	pan_lock ();
 	dialog_about (Pan.window);
-	pan_unlock ();
 }
 
 /* abfhm */
@@ -528,7 +533,7 @@ thread_articles_cb (GtkCheckMenuItem* item, gpointer data)
 static void
 rot_13_cb (GtkCheckMenuItem* item, gpointer data)
 {
-	text_set_rot13 (item->active);
+	text_rot13_selected_text_nolock ();
 }
 
 static void
@@ -571,29 +576,27 @@ download_bodies_too_toggle_cb (GtkCheckMenuItem * item, gpointer data)
 	download_bodies_too = item->active;
 }
 
-static gint
+static int
 log_button_clicked_idle (gpointer data)
 {
 	GtkWidget * old_child;
 	GtkWidget * new_child = status_ok_label;
 
-	pan_lock_unconditional ();
+	pan_lock ();
 	if (((old_child = GTK_BIN(error_button)->child)) != new_child) {
 		gtk_container_remove (GTK_CONTAINER(error_button), old_child);
 		gtk_container_add (GTK_CONTAINER(error_button), new_child);
 		gtk_widget_show (new_child);
 	}
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
 	return 0;
 }
 
-static gint
+static int
 log_button_clicked (gpointer menu, gpointer data)
 {
-	pan_lock ();
-	run_in_main_thread_nolock (log_button_clicked_idle, NULL);
-	pan_unlock ();
+	gui_queue_add (log_button_clicked_idle, NULL);
 
 	dialog_log_viewer ();
 
@@ -633,7 +636,7 @@ zoom_page_cb (gpointer menu, gpointer data)
 static void
 zoom_cb (void)
 {
-	const gint page = gui_page_get ();
+	const int page = gui_page_get_nolock ();
        	gui_set_layout (current_layout==GUI_PANED ? GUI_NOTEBOOK : GUI_PANED);
 	gui_page_set (page);
 }
@@ -728,13 +731,13 @@ static GnomeUIInfo view_menu[] =
 		'E', 0, NULL
 	},
 /*14*/	{
-		GNOME_APP_UI_TOGGLEITEM,
-		N_("_Rot13 Article Body"),
-		N_("Rot13 Article Body"),
+		GNOME_APP_UI_ITEM,
+		N_("_Rot13 Selected Text"),
+		N_("Rot13 Selected Text"),
 		rot_13_cb, NULL,
 		NULL,
 		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK,
-		'\0', 0, NULL
+		'"', GDK_CONTROL_MASK, NULL
 	},
 /*15*/	{
 		GNOME_APP_UI_TOGGLEITEM,
@@ -921,8 +924,8 @@ queue_button_clicked (GtkButton *button,
 static void
 gui_shutdown (void)
 {
-	gint i;
-	gchar * pch;
+	int i;
+	char * pch;
 	GtkWidget * w[3];
 	debug_enter ("gui_shutdown");
 
@@ -961,38 +964,40 @@ gui_shutdown (void)
 	pan_callback_remove (articlelist_selection_changed,
 	                     update_menus_cb, NULL);
 
-	g_source_remove (gui_refresh_timeout_id);
-	gui_refresh_timeout_id = 0;
+	g_source_remove (gui_refresh_timer_id);
+	gui_refresh_timer_id = 0;
+	g_source_remove (gui_queue_timer_id);
+	gui_queue_timer_id = 0;
 
 	/**
 	***  Save settings
 	**/
 
-	gnome_config_push_prefix ("/Pan/Display/");
-	gnome_config_set_bool ("show_group_pane", show_group_pane);
-	gnome_config_set_bool ("header_pane_is_threaded", header_pane_is_threaded);
-	gnome_config_set_bool ("show_header_pane", show_header_pane);
-	gnome_config_set_bool ("show_article_pane", show_article_pane);
-	gnome_config_set_bool ("navigate_cached_only", navigate_cached_only);
-	gnome_config_set_bool ("navigate_read_on_select", navigate_read_on_select);
-	gnome_config_set_bool ("collapse_group_names", collapse_group_names);
-	gnome_config_set_bool ("do_wrap", text_get_wrap());
-	gnome_config_set_bool ("use_fixed_font", text_use_fixed_font);
-	gnome_config_set_bool ("Show_All_Headers_In_Body", text_get_show_all_headers());
-	gnome_config_set_bool ("Mute_Quoted_text", text_get_mute_quoted());
+	pan_config_push_prefix ("/Pan/Display/");
+	pan_config_set_bool ("show_group_pane", show_group_pane);
+	pan_config_set_bool ("header_pane_is_threaded", header_pane_is_threaded);
+	pan_config_set_bool ("show_header_pane", show_header_pane);
+	pan_config_set_bool ("show_article_pane", show_article_pane);
+	pan_config_set_bool ("navigate_cached_only", navigate_cached_only);
+	pan_config_set_bool ("navigate_read_on_select", navigate_read_on_select);
+	pan_config_set_bool ("collapse_group_names", collapse_group_names);
+	pan_config_set_bool ("do_wrap", text_get_wrap());
+	pan_config_set_bool ("use_fixed_font", text_use_fixed_font);
+	pan_config_set_bool ("Show_All_Headers_In_Body", text_get_show_all_headers());
+	pan_config_set_bool ("Mute_Quoted_text", text_get_mute_quoted());
 
-	gnome_config_pop_prefix ();
+	pan_config_pop_prefix ();
 
-	gnome_config_set_bool ("/Pan/General/download_bodies_too",
+	pan_config_set_bool ("/Pan/General/download_bodies_too",
 	                       download_bodies_too);
-	gnome_config_set_int ("/Pan/State/viewmode", Pan.viewmode);
+	pan_config_set_int ("/Pan/State/viewmode", Pan.viewmode);
 
 	grouplist_shutdown_module ();
 
 	switch (Pan.viewmode)
 	{
 		case GUI_NOTEBOOK:
-			gnome_config_set_int ("/Pan/State/page", gui_page_get ());
+			pan_config_set_int ("/Pan/State/page", gui_page_get_nolock ());
 			break;
 
 		case GUI_PANED:
@@ -1009,22 +1014,22 @@ gui_shutdown (void)
 			switch(*layout_str)
 			{
 				case '2':
-					gnome_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
-					gnome_config_set_int ("/Pan/Geometry/hpaned", w[1]->allocation.width);
+					pan_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
+					pan_config_set_int ("/Pan/Geometry/hpaned", w[1]->allocation.width);
 					break;
 				case '4':
-					gnome_config_set_int ("/Pan/Geometry/vpaned", w[1]->allocation.height);
-					gnome_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.width);
+					pan_config_set_int ("/Pan/Geometry/vpaned", w[1]->allocation.height);
+					pan_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.width);
 					break;
 				case '5':
-					gnome_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
-					gnome_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.height + w[1]->allocation.height);
+					pan_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
+					pan_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.height + w[1]->allocation.height);
 					break;
 				case '1':
 				case '3':
 				default:
-					gnome_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
-					gnome_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.width);
+					pan_config_set_int ("/Pan/Geometry/vpaned", w[0]->allocation.height);
+					pan_config_set_int ("/Pan/Geometry/hpaned", w[0]->allocation.width);
 					break;
 			}
 	}
@@ -1032,7 +1037,7 @@ gui_shutdown (void)
 	gui_save_column_widths (Pan.group_clist, "group");
 	gui_save_column_widths (Pan.article_ctree, "thread_pane");
 	gui_save_window_size (Pan.window, "main_window");
-	gnome_config_sync();
+	pan_config_sync();
 
 	/**
 	***  Clear out the UI pieces
@@ -1123,7 +1128,7 @@ warn_about_tasks_running (void)
 	if (num_tasks != 0)
 	{
 		guint button = 0;
-		gchar *msg;
+		char *msg;
 		GtkWidget * dialog;
 
 		if (num_tasks == 1)
@@ -1189,20 +1194,20 @@ text_fill_body_changed_cb (gpointer call_obj,
 	                                call_arg!=NULL);
 }
 
-static gint
+static int
 log_list_appended_idle (gpointer data)
 {
 	GtkWidget * old_child;
 	GtkWidget * new_child = status_error_label;
 
-	pan_lock_unconditional ();
+	pan_lock ();
 	if (((old_child = GTK_BIN(error_button)->child)) != new_child)
 	{
 		gtk_container_remove (GTK_CONTAINER(error_button), old_child);
 		gtk_container_add (GTK_CONTAINER(error_button), new_child);
 		gtk_widget_show (new_child);
 	}
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
 	return 0;
 }
@@ -1211,11 +1216,7 @@ static void
 log_list_appended_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 {
 	if (((LogEntry*)call_obj)->severity== LOG_ERROR)
-	{
-		pan_lock ();
-		run_in_main_thread_nolock (log_list_appended_idle, NULL);
-		pan_unlock ();
-	}
+		gui_queue_add (log_list_appended_idle, NULL);
 }
 
 
@@ -1223,20 +1224,20 @@ log_list_appended_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 ****
 ***/
 
-static gint
+static int
 gui_set_title_idle (gpointer data)
 {
 	Server * server;
 	const Group * group = GROUP(data);
 	GString * s = g_string_new (NULL);
-	const gchar * servername = NULL;
-	const gchar * groupname = NULL;
+	const char * servername = NULL;
+	const char * groupname = NULL;
 	debug_enter ("gui_set_title_idle");
 
 	/* look for active server & group */
 	server = serverlist_get_active_server ();
 	if (server != NULL)
-		servername = server->name;
+		servername = server_get_name (server);
 	if (group != NULL)
 		groupname = group_get_readable_name (group);
 
@@ -1251,9 +1252,9 @@ gui_set_title_idle (gpointer data)
 		                   group->article_qty);
 
 	/* set the title in the UI */
-	pan_lock_unconditional ();
+	pan_lock ();
 	gtk_window_set_title (GTK_WINDOW (Pan.window), s->str);
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
 	/* cleanup */
 	g_string_free (s, TRUE);
@@ -1265,9 +1266,7 @@ gui_set_title_idle (gpointer data)
 static void
 gui_set_title (const Group * group)
 {
-	pan_lock ();
-	run_in_main_thread_nolock (gui_set_title_idle, (gpointer)group);
-	pan_unlock ();
+	gui_queue_add (gui_set_title_idle, (gpointer)group);
 }
 
 static void
@@ -1283,10 +1282,10 @@ articles_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 ****
 ***/
 
-static gint
+static int
 update_menus_idle (gpointer data)
 {
-	gint i;
+	int i;
 	GnomeUIInfo * menu = NULL;
 	const Article * article;
 	gboolean have_article;
@@ -1316,7 +1315,7 @@ update_menus_idle (gpointer data)
 	sendlater = serverlist_get_named_folder (PAN_SENDLATER);
 	sendlater_empty = sendlater!=NULL && !sendlater->article_qty;
 
-	pan_lock_unconditional ();
+	pan_lock ();
 
 	/* update view menu */
 	menu = view_menu;
@@ -1354,6 +1353,7 @@ update_menus_idle (gpointer data)
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* next thread */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* prev */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* prev thread */
+	gtk_widget_set_sensitive (menu[++i].widget, have_group && articlelist_has_prev_read()); /* prev read */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* parent */
 	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* top */
 	++i;
@@ -1409,11 +1409,11 @@ update_menus_idle (gpointer data)
 	gtk_widget_set_sensitive (menu[++i].widget, have_article); /* move thread */
 	++i;
 	gtk_widget_set_sensitive (menu[++i].widget, have_article); /* cancel */
-	gtk_widget_set_sensitive (menu[++i].widget, have_article); /* supercede */
+	gtk_widget_set_sensitive (menu[++i].widget, have_article); /* supersede */
 	++i;
 	gtk_widget_set_sensitive (menu[++i].widget, have_article); /* delete */
 	gtk_widget_set_sensitive (menu[++i].widget, have_article); /* delete thread  */
-	gtk_widget_set_sensitive (menu[++i].widget, have_article); /* delete all  */
+	gtk_widget_set_sensitive (menu[++i].widget, have_group); /* delete all  */
 
 
 	/* update file menu */
@@ -1433,10 +1433,10 @@ update_menus_idle (gpointer data)
 	gtk_widget_set_sensitive (menu[++i].widget, TRUE); /* exit */
 
 
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
 	/* update the title */
-	gui_set_title (alist_group);
+	gui_set_title_idle ((gpointer)alist_group);
 
 	debug_exit ("update_menus_idle");
 	return 0;
@@ -1445,9 +1445,7 @@ update_menus_idle (gpointer data)
 static void
 update_menus_cb (gpointer call_object, gpointer call_arg, gpointer user_data)
 {
-	pan_lock ();
-	run_in_main_thread_nolock (update_menus_idle, NULL);
-	pan_unlock ();
+	gui_queue_add (update_menus_idle, NULL);
 }
 
 static void
@@ -1471,7 +1469,7 @@ log_entry_added_cb (gpointer entry_data, gpointer unused1, gpointer unused2)
  * initialize and place the basic interface components
  *--------------------------------------------------------------------*/
 void
-gui_construct (const gchar * geometry)
+gui_construct (const char * geometry)
 {
 	debug_enter ("gui_construct");
 
@@ -1498,7 +1496,7 @@ gui_construct (const gchar * geometry)
 	}
 	else
 	{
-		gint x, y, w, h;
+		int x, y, w, h;
 		if (gnome_parse_geometry (geometry, &x, &y, &w, &h) )
 		{
 			if (x != -1)
@@ -1543,7 +1541,7 @@ gui_construct (const gchar * geometry)
 	gtk_widget_ref (groups_vbox);
 	gtk_widget_ref (articlelist_ctree);
 	gtk_widget_ref (text_box);
-        gui_set_layout (gnome_config_get_int ( "Pan/State/viewmode=1" ));
+        gui_set_layout (pan_config_get_int ( "Pan/State/viewmode=1" ));
 
 	pan_callback_add (article_get_articles_changed_callback(),
 	                  articles_changed_cb, NULL);
@@ -1596,13 +1594,15 @@ gui_construct (const gchar * geometry)
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(group_menu[2].widget),
 	                                download_bodies_too);
 
-	text_set_show_all_headers (gnome_config_get_bool("/Pan/Display/Show_All_Headers_In_Body=false"));
+	text_set_show_all_headers (pan_config_get_bool("/Pan/Display/Show_All_Headers_In_Body=false"));
 
-	text_set_mute_quoted (gnome_config_get_bool("/Pan/Display/Mute_Quoted_Text=false"));
+	text_set_mute_quoted (pan_config_get_bool("/Pan/Display/Mute_Quoted_Text=false"));
 
 	gui_set_title (NULL);
 
-	gui_refresh_timeout_id = gtk_timeout_add (2192, gui_refresh_timeout_cb, NULL); 
+	gui_refresh_timer_id = pan_timeout_add (3000, gui_refresh_timer_cb, NULL); 
+
+	gui_queue_timer_id = pan_timeout_add (160, gui_queue_timer_cb, NULL);
 
 	server_menu_update ();
 
@@ -1618,7 +1618,7 @@ gui_construct (const gchar * geometry)
 ****/
 
 void
-gui_set_layout (gint new_layout)
+gui_set_layout (int new_layout)
 {
 	switch (new_layout) {
 		case GUI_PANED: gui_paned_construct (); break;
@@ -1630,38 +1630,51 @@ gui_set_layout (gint new_layout)
 	update_menus_cb (NULL, NULL, NULL);
 }
 
-void
-gui_page_set (gint page)
+static int
+gui_page_set_idle (gpointer page_gpointer)
 {
-	GtkWidget * focus_pane = gui_page_get_pane (page);
+	int page = GPOINTER_TO_INT (page_gpointer);
+	GtkWidget * focus_pane;
+	debug_enter ("gui_page_set_idle");
 
+	pan_lock ();
+	focus_pane = gui_page_get_pane_nolock (page);
 	switch (Pan.viewmode)
 	{
 		case GUI_PANED:
-			/* small tweak: it's not very useful to focus the message pane */
+			/* blah, users hated this.  remove after string thaw.
 			if (page==ARTICLELIST_PAGE || page==MESSAGE_PAGE)
-				focus_pane = gui_page_get_pane (ARTICLELIST_PAGE);
-		    	gui_paned_page_set (page, focus_pane);
+				focus_pane = gui_page_get_pane_nolock (ARTICLELIST_PAGE); */
+		    	gui_paned_page_set_nolock (page, focus_pane);
 			break;
 
 		case GUI_NOTEBOOK:
-			gui_notebook_page_set (page, focus_pane);
+			gui_notebook_page_set_nolock (page, focus_pane);
 			break;
 
 		default:
 			pan_warn_if_reached ();
 			break;
 	}
+	pan_unlock ();
+
+	debug_exit ("gui_page_set_idle");
+	return 0;
+}
+void
+gui_page_set (int page)
+{
+	gui_queue_add (gui_page_set_idle, GINT_TO_POINTER(page));
 }
 
-static gint
-gui_page_get (void)
+static int
+gui_page_get_nolock (void)
 {
-	gint retval = 0;
+	int retval = 0;
 
 	switch (Pan.viewmode) {
-		case GUI_PANED:    retval = gui_paned_get_current_pane();    break;
-		case GUI_NOTEBOOK: retval = gui_notebook_get_current_pane(); break;
+		case GUI_PANED:    retval = gui_paned_get_current_pane_nolock();    break;
+		case GUI_NOTEBOOK: retval = gui_notebook_get_current_pane_nolock(); break;
 		default: pan_warn_if_reached(); break;
 	}
 
@@ -1669,7 +1682,7 @@ gui_page_get (void)
 }
 
 static GtkWidget*
-gui_page_get_pane (gint page)
+gui_page_get_pane_nolock (int page)
 {
 	GtkWidget * retval = NULL;
 
@@ -1684,7 +1697,7 @@ gui_page_get_pane (gint page)
 }
 
 static void
-gui_page_activate_nolock (gint page)
+gui_page_activate_nolock (int page)
 {
 	switch (page)
 	{
@@ -1697,17 +1710,19 @@ gui_page_activate_nolock (gint page)
 		}
 		case ARTICLELIST_PAGE:
 		{
-			GPtrArray * articles = articlelist_get_selected_articles_nolock ();
-			if (articles->len > 0) {
-				Article * a = ARTICLE(g_ptr_array_index(articles,0));
-				text_set_from_article (a);
-			}
+			GPtrArray * articles;
+
+			gui_queue_add ((GSourceFunc) articlelist_read_selected, NULL);
+
+			/* also download any others that are selected */
+			articles = articlelist_get_selected_articles_nolock ();
 			if (articles->len > 1) {
 				Group * g = ARTICLE(g_ptr_array_index(articles,0))->group;
 				g_ptr_array_remove_index (articles, 0);
 				queue_add (TASK(task_bodies_new (g, articles)));
 			}
 			g_ptr_array_free (articles, TRUE);
+
 			break;
 		}
 		case MESSAGE_PAGE:
@@ -1719,9 +1734,9 @@ gui_page_activate_nolock (gint page)
 }
 
 static void
-gui_page_change (int change)
+gui_page_change_nolock (int change)
 {
-	gint page = gui_page_get() + change;
+	int page = gui_page_get_nolock () + change;
 
 	if (0<=page && page<=2)
 		gui_page_set (page);
@@ -1741,7 +1756,7 @@ gui_key_press_cb (GtkWidget      * widget,
 	switch (event->keyval)
 	{
 		case GDK_Q: case GDK_q:
-			gui_page_change (-1);
+			gui_page_change_nolock (-1);
 			break;
 
 		case GDK_Delete:
@@ -1752,8 +1767,8 @@ gui_key_press_cb (GtkWidget      * widget,
 			break;
 
 		case GDK_Return:
-			gui_page_activate_nolock (gui_page_get());
-			gui_page_change (1);
+			gui_page_activate_nolock (gui_page_get_nolock ());
+			gui_page_change_nolock (1);
 			break;
 
 		case GDK_space: case GDK_KP_Space:
@@ -1773,19 +1788,19 @@ gui_key_press_cb (GtkWidget      * widget,
 
 void
 gui_restore_column_widths (GtkWidget     * clist,
-                           const gchar   * type)
+                           const char   * type)
 {
 	int i;
 	GtkCList * list = GTK_CLIST (clist);
 	const int cols = list->columns;
 	int * widths = g_new (int, cols);
-	GString * buf = g_string_new (NULL);
+	char buf[1024];
 	debug_enter ("gui_restore_column_widths");
 
 	/* get width from config... */
 	for (i=0; i!=cols; ++i) {
-		g_string_sprintf (buf, "/Pan/Geometry/%s_column_%d", type, i);
-		widths[i] = gnome_config_get_int (buf->str);
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
+		widths[i] = pan_config_get_int (buf);
 	}
 
 	/* set ui.. */
@@ -1796,28 +1811,28 @@ gui_restore_column_widths (GtkWidget     * clist,
 	pan_unlock();
 
 	/* cleanup */
-	g_string_free (buf, TRUE);
 	g_free (widths);
 	debug_exit ("gui_restore_column_widths");
 }
 
+
 void
 gui_save_column_widths (GtkWidget    * clist,
-                        const gchar  *type)
+                        const char   * type)
 {
 	int i;
 	int cols = GTK_CLIST (clist)->columns;
 	GtkCList * list = GTK_CLIST (clist);
-	GString * buf = g_string_new (NULL);
+	char buf[1024];
 	
 	for (i=0; i<cols; i++)
 	{
-		g_string_sprintf (buf, "/Pan/Geometry/%s_column_%d", type, i);
-		gnome_config_set_int (buf->str, (int)list->column[i].width);
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
+		pan_config_set_int (buf, (int)list->column[i].width);
 	}
 
-	g_string_free (buf, TRUE);
-	gnome_config_sync ();
+
+	pan_config_sync ();
 }
 
 /**
@@ -1826,13 +1841,13 @@ gui_save_column_widths (GtkWidget    * clist,
 
 void
 gui_restore_window_size (GtkWidget     * widget,
-                         const gchar   * key)
+                         const char    * key)
 {
-	gint x, y, w, h;
-	GString * buf;
-	gint screen_w = gdk_screen_width ();
-	gint screen_h = gdk_screen_height ();
-	const gint fuzz = 10;
+	int x, y, w, h;
+	char buf[1024];
+	int screen_w = gdk_screen_width ();
+	int screen_h = gdk_screen_height ();
+	const int fuzz = 10;
 
 	/* sanity clause */
 	g_return_if_fail (GTK_IS_WIDGET(widget));
@@ -1840,15 +1855,14 @@ gui_restore_window_size (GtkWidget     * widget,
 	g_return_if_fail (is_nonempty_string(key));
 
 	/* get the dimensions */
-	buf = g_string_new (NULL);
-	g_string_sprintf (buf, "/Pan/Geometry/%s_x", key);
-	x = gnome_config_get_int (buf->str);
-	g_string_sprintf (buf, "/Pan/Geometry/%s_y", key);
-	y = gnome_config_get_int (buf->str);
-	g_string_sprintf (buf, "/Pan/Geometry/%s_width", key);
-	w = gnome_config_get_int (buf->str);
-	g_string_sprintf (buf, "/Pan/Geometry/%s_height", key);
-	h = gnome_config_get_int (buf->str);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
+	x = pan_config_get_int (buf);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
+	y = pan_config_get_int (buf);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_width", key);
+	w = pan_config_get_int (buf);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_height", key);
+	h = pan_config_get_int (buf);
 
 	/* move & resize */
 	if ((0<x && x<screen_w-fuzz) && (0<y && y<screen_h-fuzz))
@@ -1856,37 +1870,38 @@ gui_restore_window_size (GtkWidget     * widget,
 	if (w>0 && h>0)
 		gtk_window_set_default_size (GTK_WINDOW(widget), w, h);
 
-	/* cleanup */
-	g_string_free (buf, TRUE);
+
+
+
 }
  
 void
 gui_save_window_size (GtkWidget     * widget,
-                      const gchar   * key)
+                      const char    * key)
 {
-	GString * buf;
-	gint x, y, w, h;
+	char buf[1024];
+	int x, y, w, h;
 
 	/* sanity clause */
 	g_return_if_fail (GTK_IS_WIDGET(widget));
 	g_return_if_fail (widget->window!=NULL);
 	g_return_if_fail (is_nonempty_string(key));
 
-	buf = g_string_new (NULL);
 	gdk_window_get_position (widget->window, &x, &y);
 	gdk_window_get_size (widget->window, &w, &h);
 
-	g_string_sprintf (buf, "/Pan/Geometry/%s_x", key);
-	gnome_config_set_int (buf->str, x);
-	g_string_sprintf (buf, "/Pan/Geometry/%s_y", key);
-	gnome_config_set_int (buf->str, y);
-	g_string_sprintf (buf, "/Pan/Geometry/%s_width", key);
-	gnome_config_set_int (buf->str, w);
-	g_string_sprintf (buf, "/Pan/Geometry/%s_height", key);
-	gnome_config_set_int (buf->str, h);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
+	pan_config_set_int (buf, x);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
+	pan_config_set_int (buf, y);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_width", key);
+	pan_config_set_int (buf, w);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_height", key);
+	pan_config_set_int (buf, h);
+
+	pan_config_sync ();
+
 
-	g_string_free (buf, TRUE);
-	gnome_config_sync ();
 }
 
 /*****
@@ -1897,7 +1912,7 @@ static int connection_size = -1;
 
 static gboolean _is_connecting = FALSE;
 
-static gint
+static int
 gui_refresh_connection_label_idle (gpointer data)
 {
 	char str[128];
@@ -1920,13 +1935,13 @@ gui_refresh_connection_label_idle (gpointer data)
 	}
 
 	/* update the tip */
-	pan_lock_unconditional ();
+	pan_lock ();
 	if (GTK_IS_LABEL(connection_qty_label)) {
 		gtk_label_set_text (GTK_LABEL(connection_qty_label), str);
 	if (GTK_IS_BUTTON(connection_qty_button))
 		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), connection_qty_button, tip, "");
 	}
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
 	return 0;
 }
@@ -1937,10 +1952,7 @@ gui_set_connecting_flag (gboolean is_connecting)
 	if (_is_connecting != is_connecting)
 	{
 		_is_connecting = is_connecting;
-
-		pan_lock ();
-		run_in_main_thread_nolock (gui_refresh_connection_label_idle, NULL);
-		pan_unlock ();
+		gui_queue_add (gui_refresh_connection_label_idle, NULL);
 	}
 }
 
@@ -1951,10 +1963,7 @@ gui_set_connection_size (int size)
 	{
 		connection_size = size;
 		log_add_va (LOG_INFO, _("News server connection count: %d"), size);
-
-		pan_lock ();
-		run_in_main_thread_nolock (gui_refresh_connection_label_idle, NULL);
-		pan_unlock ();
+		gui_queue_add (gui_refresh_connection_label_idle, NULL);
 	}
 }
 
@@ -1965,7 +1974,7 @@ gui_inc_connection_size (int inc)
 }
 
 static int
-gui_refresh_timeout_cb (gpointer user_data)
+gui_refresh_timer_cb (gpointer user_data)
 {
 	/* update KBps */
 	if (1)
@@ -2000,9 +2009,7 @@ gui_refresh_timeout_cb (gpointer user_data)
 		}
 	}
 
-	pan_lock_unconditional ();
-	run_in_main_thread_nolock (gui_refresh_connection_label_idle, NULL);
-	pan_unlock_unconditional ();
+	gui_queue_add (gui_refresh_connection_label_idle, NULL);
 
 	return 1;
 }
@@ -2032,14 +2039,14 @@ gui_set_queue_size (guint running, guint size)
 	pan_unlock();
 }
 
-static gint
+static int
 gui_add_status_item_idle (gpointer p)
 {
 	int i;
 	StatusItem * status;
 	debug_enter ("gui_add_status_item");
 
-	pan_lock_unconditional ();
+	pan_lock ();
 
 	for (i=0; i<VIEW_QTY; ++i)
 		if (views[i].status_item == NULL)
@@ -2053,19 +2060,19 @@ gui_add_status_item_idle (gpointer p)
 	}
 
 	pan_object_unref (PAN_OBJECT(status)); /* ref in item_active_changed_cb */
-	pan_unlock_unconditional ();
+	pan_unlock ();
 	debug_exit ("gui_add_status_item");
 	return 0;
 }
 
-static gint
+static int
 gui_remove_status_item_idle (gpointer p)
 {
 	int i;
 	StatusItem * status;
 	debug_enter ("gui_remove_status_item");
 
-	pan_lock_unconditional ();
+	pan_lock ();
 
 	status = STATUS_ITEM(p);
 	for (i=0; i<VIEW_QTY; ++i)
@@ -2078,7 +2085,7 @@ gui_remove_status_item_idle (gpointer p)
 		pan_object_unref (PAN_OBJECT(status)); /* ref in set_item_idle */
 	}
 
-	pan_unlock_unconditional ();
+	pan_unlock ();
 	pan_object_unref (PAN_OBJECT(status)); /* ref in item_active_changed_cb */
 	debug_exit ("gui_remove_status_item");
 	return 0;
@@ -2087,7 +2094,7 @@ gui_remove_status_item_idle (gpointer p)
 static void
 turn_off_status_item_views_nolock (void)
 {
-	gint i;
+	int i;
 	debug_enter ("turn_off_status_item_views");
 
 	for (i=0; i<VIEW_QTY; ++i) {
@@ -2109,14 +2116,20 @@ status_item_active_changed_cb (gpointer call_obj, gpointer call_arg, gpointer us
 	gboolean is_active = GPOINTER_TO_INT(call_arg) != 0;
 	debug_enter ("status_item_active_changed_cb");
 
-	if (is_active) {
-		pan_object_ref (PAN_OBJECT(item));
-		run_in_main_thread_nolock (gui_add_status_item_idle, item);
-	}
-	else {
-		pan_object_ref (PAN_OBJECT(item));
-		run_in_main_thread_nolock (gui_remove_status_item_idle, item);
-	}
+	/**
+	***  Ref the item to make sure it's still alive when the
+	***  idle func is run.  The idle func unrefs it to balance
+	***  this out.
+	**/
+
+	pan_object_ref (PAN_OBJECT(item));
+
+	/**
+	***  Push the GUI work to the main thread
+	**/
+
+	gui_queue_add (is_active ? gui_add_status_item_idle : gui_remove_status_item_idle, item);
+
 
 	debug_exit ("status_item_active_changed_cb");
 }
@@ -2128,7 +2141,7 @@ status_item_active_changed_cb (gpointer call_obj, gpointer call_arg, gpointer us
 static void
 pan_widget_set_font_on_realize (GtkObject * o, gpointer user_data)
 {
-	gchar * font_name = gtk_object_get_data (o, "font_name");
+	char * font_name = gtk_object_get_data (o, "font_name");
 
 	/* sanity clause */
 	g_return_if_fail (GTK_IS_WIDGET(o));
@@ -2167,3 +2180,67 @@ pan_widget_set_font (GtkWidget * w, const char * font_name)
 		gtk_signal_connect (GTK_OBJECT(w), "realize", pan_widget_set_font_on_realize, NULL);
 	}
 }
+
+/*****
+******  This is for delegating small, async GUI updates to the main thread
+******  to reduce the number of threads contending for a GUI lock.
+*****/
+
+static GStaticMutex gui_queue_mutex = G_STATIC_MUTEX_INIT;
+
+static GSList * gui_queue_slist = NULL;
+
+typedef struct
+{
+	GSourceFunc func;
+	gpointer user_data;
+}
+GuiQueueItem;
+
+static int
+gui_queue_timer_cb (gpointer user_data)
+{
+	GSList * list;
+
+	/* get the work list */
+	g_static_mutex_lock (&gui_queue_mutex);
+	list = gui_queue_slist;
+	gui_queue_slist = NULL;
+	g_static_mutex_unlock (&gui_queue_mutex);
+
+	if (list != NULL)
+	{
+		GSList * l;
+
+		/* gui_queue_slist is LIFO for efficiency of appending nodes;
+		 * reverse the nodes to make list FIFO */
+		list = g_slist_reverse (list);
+
+		/* run the work functions */
+		for (l=list; l!=NULL; l=l->next)
+		{
+			GuiQueueItem * queue_item = (GuiQueueItem*) l->data;
+			(*queue_item->func)(queue_item->user_data);
+			g_free (queue_item);
+		}
+		g_slist_free (list);
+	}
+
+	return 1;
+}
+
+void
+gui_queue_add (GSourceFunc func, gpointer user_data)
+{
+	GuiQueueItem * item;
+
+	/* create the new queue item */	
+	item = g_new (GuiQueueItem, 1);
+	item->func = func;
+	item->user_data = user_data;
+
+	/* add it to the queue */
+	g_static_mutex_lock (&gui_queue_mutex);
+	gui_queue_slist = g_slist_prepend (gui_queue_slist, item);
+	g_static_mutex_unlock (&gui_queue_mutex);
+}
