@@ -25,6 +25,7 @@
 #include <pan/base/debug.h>
 #include <pan/base/log.h>
 #include <pan/base/newsrc-port.h>
+#include <pan/base/pan-config.h>
 #include <pan/base/serverlist.h>
 #include <pan/base/util-file.h>
 #include <pan/filters/filter-manager.h>
@@ -79,7 +80,7 @@ struct poptOption options[] = {
 	  N_("GEOMETRY") },
 	{ "mute", '\0', 0, &mute, 0,
 	  N_("Send outgoing messages to stdout instead of smtp/nntp servers for debugging purposes."), NULL },
-	{ "debug-acache", '\0', 0, &debug_acache, 0,
+	{ "debug-cache", '\0', 0, &debug_acache, 0,
 	  N_("Turn on article cache debugging messages."), NULL },
 	{ "debug-socket-input", '\0', 0, &debug_socket_input, 0,
 	  N_("Turn on socket input debugging messages."), NULL },
@@ -175,8 +176,8 @@ main (int argc, char *argv[])
 		g_message ("Running in mute mode: messages will be sent to "
 		           "stdout instead of smtp/nntp server");
 
-	queue_init (gnome_config_get_int("/Pan/General/Max_Connections_Total=4"),
-	            gnome_config_get_int("/Pan/General/Max_Task_Tries_Total=4"));
+	queue_init (pan_config_get_int("/Pan/General/Max_Connections_Total=4"),
+	            pan_config_get_int("/Pan/General/Max_Task_Tries_Total=4"));
 
 	/* if it's a new user, pop up the newuser dialog.
 	   Otherwise show the main window. */
