@@ -224,6 +224,7 @@ queue_init (int socket_qty, int tries)
 	pthread_mutex_init (&cond_mutex, NULL);
 	pthread_mutex_init (&todo_mutex, NULL);
 	pthread_mutex_init (&sock_lock, NULL);
+	pthread_mutex_init (&task_lock, NULL);
 
 	pan_callback_add (server_get_online_status_changed_callback(),
 	                  server_online_status_changed_cb, NULL);
@@ -265,6 +266,10 @@ real_queue_set_paused (gint p)
 		}
 		pthread_mutex_unlock (&sock_lock);
 	}
+	else
+	{
+		queue_wakeup ();
+	}
 }
 
 
@@ -622,7 +627,7 @@ queue_set_task_status (Task* task, QueueTaskStatus status)
 static void
 queue_new_todo (QueueTodoAction action, Task* task, int i1, int i2)
 {
-	QueueTodo *todo = g_new0 (QueueTodo, 1);
+	QueueTodo *todo = g_new (QueueTodo, 1);
 	todo->task = task;
 	todo->index_1 = i1;
 	todo->index_2 = i2;
@@ -670,15 +675,22 @@ queue_run_thread (void* vp)
 	/* clean up gui */
 	status_item_set_active (STATUS_ITEM(task), FALSE);
 
-	/* clean up socket */
+	/* clean up socket.
+	   We throw away any socket of a failed or aborted task,
+	   just in case the read buffer has junk in there. */
 	aborted = queue_get_task_status(task) == QUEUE_TASK_STATUS_ABORTING;
-	if (aborted)
+	if (task->sock!=NULL && (aborted || result!=TASK_SUCCESS))
 		task->sock->error = TRUE;
 	socket_cleanup (task);
+
 	/* clean up the task */
 	task->is_running = FALSE;
 
-	if (result==TASK_SUCCESS) /* success */
+	if (result==TASK_SUCCESS) 
+	{
+		queue_task_remove (task);
+	}
+	else if (result==TASK_FAIL_HOPELESS && remove_failed_tasks) 
 	{
 		queue_task_remove (task);
 	}
@@ -824,7 +836,7 @@ queue_get_socket_for_task (Task *task)
 		}
 		else	
 		{
-			const char * errmsg = NULL;
+			char * errmsg = NULL;
 
 			/* open a new socket.. */
 			PanSocket *sock = NULL;
@@ -835,20 +847,25 @@ queue_get_socket_for_task (Task *task)
 			pan_socket_set_nntp_auth (sock, server->need_auth, server->username, server->password); 
 
 			debug1 (DEBUG_QUEUE, "new socket %p", sock);
-			if (sock->sockfd < 0)
-				errmsg = "Unable to connect to server";
+			if (sock == NULL || sock->sockfd < 0)
+			{
+				if (is_nonempty_string (server->address))
+					errmsg = g_strdup_printf (_("Unable to connect to server %s:%d\n"), server->address, server->port);
+				else
+					errmsg = g_strdup (_("No server address specified in the Preferences"));
+			}
 			else /* handshake */
 			{
 				StatusItem * status = STATUS_ITEM(task);
 				gboolean posting_ok = FALSE;
 				gint val = nntp_handshake (status, sock, &posting_ok);
 				if (val != TASK_SUCCESS)
-					errmsg = (const gchar*)(g_slist_last(status->errors)->data);
+					errmsg = g_strdup(g_slist_last(status->errors)->data);
 			}
 			if (errmsg!=NULL)
 			{
 				log_add (LOG_ERROR, errmsg);
-				g_warning(errmsg);
+				g_free (errmsg);
 				if (sock)
 					pan_object_unref (PAN_OBJECT(sock));
 				sock = NULL;
@@ -864,7 +881,7 @@ queue_get_socket_for_task (Task *task)
 				debug1 (DEBUG_QUEUE, "%p logged in", sock);
 
 				/* looks like we got a new socket okay... */
-				si = g_new0 (SocketInfo, 1);
+				si = g_new (SocketInfo, 1);
 				si->socket = sock;
 				si->server = server;
 				si->task = task;
@@ -1017,10 +1034,7 @@ socket_cleanup (Task* task)
 		else
 		{
 			/* otherwise throw it away */
-			pan_socket_putline (socket, "ccc\r\n");
 			nntp_disconnect (NULL, socket);
-			close (socket->sockfd);
-			socket->sockfd = 0;
 			pan_object_unref (PAN_OBJECT(socket));
 			gui_inc_connection_size (-1);
 			sockets = g_slist_remove (sockets, info);
