@@ -510,9 +510,11 @@ filters_changed_cb (gpointer a, gpointer b, gpointer c)
 static GtkWidget*
 create_filter_tab (RuleEditDialog * d)
 {
-	GtkWidget * h;
-	GtkWidget * w;
-	GtkWidget * top;
+	GtkWidget     * h;
+	GtkWidget     * w;
+	GtkWidget     * top;
+	GtkWidget     * text_parent;
+	GtkPolicyType   policy = GTK_POLICY_AUTOMATIC;
 
 	top = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width (GTK_CONTAINER(top), GNOME_PAD);
@@ -523,6 +525,11 @@ create_filter_tab (RuleEditDialog * d)
 	/* filter view */
 	d->filter_text = gtk_text_new (NULL, NULL);
 
+	text_parent = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(text_parent), 
+		policy, policy);
+	gtk_container_add (GTK_CONTAINER(text_parent), d->filter_text);
+
 	/* "edit filters" button */
 	h = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width (GTK_CONTAINER(h), GNOME_PAD_SMALL);
@@ -541,7 +548,7 @@ create_filter_tab (RuleEditDialog * d)
 	gtk_box_pack_start (GTK_BOX(h), gtk_label_new("      "), FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(h), d->filter_edit_filters_button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX(top), h, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX(top), d->filter_text, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX(top), text_parent, TRUE, TRUE, 0);
 
 	/* listen for changes to filter manager */
 	pan_callback_add (filter_manager_get_filters_changed_callback(),
@@ -791,9 +798,10 @@ rule_from_newsgroup_tab (RuleEditDialog * d, Rule * r)
 		r->group_type = RULE_GROUP_LIST;
 		r->group_list = g_ptr_array_new ();
 		for (i=0; i!=apply->rows; ++i) {
-			gchar * text = NULL;
-			gtk_clist_get_text (apply, i, 0, &text);
-			g_ptr_array_add (r->group_list, g_strdup(text));
+			Group *g = GROUP(gtk_clist_get_row_data (apply, i));
+			if (g) {
+				g_ptr_array_add (r->group_list, g_strdup(g->name));
+			}
 		}
 	}
 }
