@@ -43,11 +43,9 @@ struct _StatusItemView
 
 	/* private fields */  
 	guint signal_id_progress_set_percent;
-	guint signal_id_progress_init_steps;
-	guint signal_id_progress_next_step;
 	guint signal_id_status;
 	guint signal_id_error;
-	gchar * last_status;
+	GString * last_status;
 	StatusItem * item;
 	GtkWidget * progress;
 	GtkWidget * eventbox;
@@ -65,7 +63,7 @@ GtkType       status_item_view_get_type            (void);
 
 GtkWidget*    status_item_view_new                 (void);
 
-void          status_item_view_set_item            (StatusItemView  *view,
+void          status_item_view_set_item_nolock     (StatusItemView  *view,
                                                     StatusItem      *item);
 
 
