@@ -1,6 +1,6 @@
 /*
  * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@superpimp.org)
+ * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
 #ifndef __TASK_GROUP_COUNT__H__
 #define __TASK_GROUP_COUNT__H__
 
-#include "task.h"
-#include "group.h"
+#include <pan/base/group.h>
+#include <pan/task.h>
 
 #define TASK_GROUP_COUNT(obj) ((TaskGroupCount*)(obj))
 
