@@ -25,15 +25,16 @@
 
 #include <config.h>
 
-#include <gnome.h>
+#include <libgnomeui/libgnomeui.h>
 
+#include <pan/base/argset.h>
 #include <pan/base/debug.h>
+#include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
 #include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/queue.h>
-#include <pan/status-item-view.h>
 #include <pan/task-manager.h>
 #include <pan/util.h>
 
@@ -70,44 +71,12 @@ typedef struct
 }
 ManagerUI;
 
-/*********************
-**********************  Private Function Prototypes
-*********************/
-
-/*********************
-**********************  Constants
-*********************/
-
-/***********
-************  Extern
-***********/
-
-/***********
-************  Public
-***********/
-
-/***********
-************  Private
-***********/
-
 /*********************
 **********************  Variables
 *********************/
 
-/***********
-************  Extern
-***********/
-
 extern GtkTooltips * ttips;
 
-/***********
-************  Public
-***********/
-
-/***********
-************  Private
-***********/
-
 static ManagerUI * manager_ui = NULL;
 
 /*********************
@@ -343,18 +312,20 @@ static void
 clear_queue_clicked_cb (GtkButton   * b,
                         gpointer      user_data)
 {
-	GSList * l;
-	GSList * queue;
+	guint i;
+	GPtrArray * tasks;
 	debug_enter ("clear_queue_clicked_cb");
 
-	queue = g_slist_copy ((GSList*)queue_get_tasks());
-	for (l=queue; l!=NULL; l=l->next) {
-		Task * task = TASK(l->data);
+	tasks = queue_get_tasks ();
+	for (i=0; i<tasks->len; ++i) {
+		Task * task = TASK(g_ptr_array_index (tasks,i));
 		if (queue_get_task_status(task)==QUEUE_TASK_STATUS_QUEUED)
 			queue_task_remove(task);
 	}
 
-	g_slist_free (queue);
+	/* cleanup */
+	pan_g_ptr_array_foreach (tasks, (GFunc)pan_object_unref, NULL);
+	g_ptr_array_free (tasks, TRUE);
 
 	debug_exit ("clear_queue_clicked_cb");
 }
@@ -404,7 +375,8 @@ get_secs_left (const Task * task)
 
 static void
 get_percent_str (const Task   * task,
-                 gchar        * buf)
+                 gchar        * buf,
+                 gint           buf_size)
 {
 	int percent = 0;
 
@@ -414,12 +386,13 @@ get_percent_str (const Task   * task,
 	if (percent == 0)
 		*buf = '\0';
 	else
-		sprintf (buf, "%d%%", percent);
+		g_snprintf (buf, buf_size, "%d%%", percent);
 }
 
 static void
 get_xfer_str (const Task    * task,
-              gchar         * buf)
+              gchar         * buf,
+              gint            buf_size)
 {
 	if (!task->sock)
 	{
@@ -432,13 +405,14 @@ get_xfer_str (const Task    * task,
 		const int m = (secs_left % 3600) / 60;
 		const int s = secs_left % 60;
 		const double KBps = pan_socket_get_xfer_rate_KBps (task->sock);
-		sprintf (buf, _("%.2f KB/s, %d:%02d:%02d remaining"), KBps, h, m, s);
+		g_snprintf (buf, buf_size, _("%.2f KB/s, %d:%02d:%02d remaining"), KBps, h, m, s);
 	}
 }
 
 static void
 get_status_str (const Task     * task,
-                gchar          * buf)
+                gchar          * buf,
+                gint             buf_size)
 {
 	const char* status_str;
 	QueueTaskStatus status = queue_get_task_status(task);
@@ -469,11 +443,11 @@ get_status_str (const Task     * task,
 	}
 
 	if (!task->server->online && status==QUEUE_TASK_STATUS_QUEUED)
-		sprintf (buf, _("Offline"));
+		g_snprintf (buf, buf_size, _("Offline"));
 	else if (!task->tries)
-		sprintf (buf, "%s", status_str);
+		g_snprintf (buf, buf_size, "%s", status_str);
 	else
-		sprintf (buf, _("%s (%d tries)"), status_str, task->tries);
+		g_snprintf (buf, buf_size, _("%s (%d tries)"), status_str, task->tries);
 }
 
 
@@ -496,9 +470,9 @@ add_task_to_list (ManagerUI     * ui,
 	text[3] = xferbuf;
 	text[4] = description;
 
-	get_xfer_str (task, xferbuf);
-	get_status_str (task, statusbuf);
-	get_percent_str (task, percentbuf);
+	get_xfer_str (task, xferbuf, sizeof(xferbuf));
+	get_status_str (task, statusbuf, sizeof(statusbuf));
+	get_percent_str (task, percentbuf, sizeof(percentbuf));
 
 	pan_lock ();
 	row = gtk_clist_insert (list, index, text);
@@ -512,10 +486,16 @@ add_task_to_list (ManagerUI     * ui,
 static void
 ui_populate (ManagerUI *ui)
 {
-	int i;
-	const GSList* l = NULL;
-	for (i=0, l=queue_get_tasks(); l!=NULL; l=l->next, ++i)
-		add_task_to_list (ui, l->data, i);
+	guint i;
+	GPtrArray * tasks;
+
+	tasks = queue_get_tasks ();
+	for (i=0; i<tasks->len; ++i)
+		add_task_to_list (ui, TASK(g_ptr_array_index(tasks,i)), i);
+
+	/* cleanup */
+	pan_g_ptr_array_foreach (tasks, (GFunc)pan_object_unref, NULL);
+	g_ptr_array_free (tasks, TRUE);
 }
 
 static void
@@ -541,42 +521,50 @@ queue_row_move_cb (GtkCList   * clist,
 ***  Queue Changes
 **/
 
-static int
-task_added_cb (gpointer call_obj,
-               gpointer call_arg,
-               gpointer client_arg)
+static int                                                                      
+task_added_idle (gpointer argset_gpointer)                                      
+{                                                                               
+        ArgSet * argset = (ArgSet*) argset_gpointer;                            
+        ManagerUI * ui = (ManagerUI*) argset_get (argset, 0);                   
+        Task * task = (Task*) argset_get (argset, 1);                           
+        const int index = GPOINTER_TO_INT (argset_get (argset, 2));             
+
+        /* insert the task */                                                   
+        add_task_to_list (ui, task, index);                                     
+
+        /* cleanup */                                                           
+        pan_object_unref (PAN_OBJECT(task));                                    
+        argset_free (argset);                                                   
+        return 0;                                                               
+}                                                                               
+
+static void
+task_added_cb (gpointer call_obj, gpointer call_arg, gpointer client_arg)
 {
 	Task * task = TASK(call_obj);
 	int index = GPOINTER_TO_INT(call_arg);
 	ManagerUI * ui = (ManagerUI*) client_arg;
 
-	add_task_to_list (ui, task, index);
-
-	return 0;
+	pan_object_ref (PAN_OBJECT(task));
+	gui_queue_add (task_added_idle, argset_new3 (ui, task, GINT_TO_POINTER(index)));
 }
 
-static int
-task_removed_cb (gpointer call_obj,
-                 gpointer call_arg,
-                 gpointer client_arg)
+static void
+task_removed_cb (gpointer call_obj, gpointer call_arg, gpointer client_arg)
 {
 	Task * task = TASK(call_obj);
 	int index = GPOINTER_TO_INT(call_arg);
 	ManagerUI * ui = (ManagerUI*) client_arg;
 	GtkCList * list = GTK_CLIST(ui->list);
 
-	g_return_val_if_fail (gtk_clist_get_row_data(list,index)==task, 0);
+	g_return_if_fail (gtk_clist_get_row_data(list,index)==task);
 	pan_lock ();
 	gtk_clist_remove (list, index);
 	pan_unlock ();
-
-	return 0;
 }
 
-static int
-task_moved_cb (gpointer call_obj,
-               gpointer call_arg,
-               gpointer client_arg)
+static void
+task_moved_cb (gpointer call_obj, gpointer call_arg, gpointer client_arg)
 {
 	ManagerUI * ui = (ManagerUI*) client_arg;
 	int old_row = ((int*)call_arg)[0];
@@ -587,8 +575,6 @@ task_moved_cb (gpointer call_obj,
 	gtk_clist_row_move (GTK_CLIST(ui->list), old_row, new_row);
 	ui->moved_signal_id = gtk_signal_connect (GTK_OBJECT(ui->list), "row_move", queue_row_move_cb, ui);
 	pan_unlock ();
-
-	return 0;
 }
 
 static gboolean
@@ -637,17 +623,22 @@ gui_key_press_cb (GtkWidget      * widget,
 }
 
 static void
-update_titlebar_lock_unconditional (ManagerUI * ui)
+update_titlebar_lock (ManagerUI * ui)
 {
 	int running, queued, failed;
-	const GSList* l;
+	guint i;
+	GPtrArray * tasks;
 	gchar* pch;
 
 	g_return_if_fail (ui!=NULL);
 
+	/* get information about the tasks */
 	running = queued = failed = 0;
-	for (l=queue_get_tasks(); l!=NULL; l=l->next) {
-		QueueTaskStatus status = queue_get_task_status (TASK(l->data));
+	tasks = queue_get_tasks ();
+	for (i=0; i<tasks->len; ++i) {
+		Task * t = TASK(g_ptr_array_index(tasks,i));
+		QueueTaskStatus status = queue_get_task_status (t);
+	
 		if (status==QUEUE_TASK_STATUS_QUEUED)
 			++queued;
 		else if (status==QUEUE_TASK_STATUS_RUNNING)
@@ -656,16 +647,23 @@ update_titlebar_lock_unconditional (ManagerUI * ui)
 			++failed;
 	}
 
+	/* create a titlebar */
 	if (failed)
 		pch = g_strdup_printf (_("Pan %s Task Manager (%d Queued, %d Running, %d Failed)"), VERSION, queued, running, failed);
 	else if (queued || running)
 		pch = g_strdup_printf (_("Pan %s Task Manager (%d Queued, %d Running)"), VERSION, queued, running);
 	else 
 		pch = g_strdup_printf (_("Pan %s Task Manager"), VERSION);
-	pan_lock_unconditional ();
+
+	/* show the titlebar */
+	pan_lock ();
 	gtk_window_set_title (GTK_WINDOW(ui->window), pch);
-	pan_unlock_unconditional ();
+	pan_unlock ();
 	g_free (pch);
+
+	/* cleanup */
+	pan_g_ptr_array_foreach (tasks, (GFunc)pan_object_unref, NULL);
+	g_ptr_array_free (tasks, TRUE);
 }
 
 static int
@@ -678,7 +676,7 @@ periodic_update_timer (gpointer user_data)
 	debug0 (DEBUG_QUEUE, "task manager periodic ui refresh");
 
 	/* update the ui */
-	pan_lock_unconditional ();
+	pan_lock ();
 	gtk_clist_freeze (clist);
 	for (row=0; row<clist->rows; ++row)
 	{
@@ -692,12 +690,12 @@ periodic_update_timer (gpointer user_data)
 			pan_object_ref (PAN_OBJECT(task));
 
 			/* have the index, now update it */
-			get_status_str (task, buf);
+			get_status_str (task, buf, sizeof(buf));
 			gtk_clist_set_text (clist, row, 0, buf);
-			get_percent_str (task, buf);
+			get_percent_str (task, buf, sizeof(buf));
 			gtk_clist_set_text (clist, row, 1, buf);
 			gtk_clist_set_text (clist, row, 2, task->server->name);
-			get_xfer_str (task, buf);
+			get_xfer_str (task, buf, sizeof(buf));
 			gtk_clist_set_text (clist, row, 3, buf);
 			pch = status_item_describe(STATUS_ITEM(task));
 			gtk_clist_set_text (clist, row, 4, pch);
@@ -709,10 +707,11 @@ periodic_update_timer (gpointer user_data)
 	}
 
 	gtk_clist_thaw (clist);
-	pan_unlock_unconditional ();
+	pan_unlock ();
 
-	if (ui->titlebar_needs_refresh) {
-		update_titlebar_lock_unconditional (ui);
+	if (ui->titlebar_needs_refresh)
+	{
+		update_titlebar_lock (ui);
 		ui->titlebar_needs_refresh = FALSE;
 	}
 
@@ -891,7 +890,7 @@ task_manager_build_ui (GtkWidget * window)
 	pan_callback_add (queue_task_moved, task_moved_cb, ui);
 
 	ui->titlebar_needs_refresh = TRUE;
-	ui->timeout_id = gtk_timeout_add (2000, periodic_update_timer, ui);
+	ui->timeout_id = pan_timeout_add (2000, periodic_update_timer, ui);
 	ui->top = vbox;
         gtk_signal_connect (GTK_OBJECT(ui->top), "destroy", ui_destroy_cb, ui);
 	return ui;
@@ -901,13 +900,10 @@ task_manager_build_ui (GtkWidget * window)
 ***  Titlebar dirty
 **/
 
-static int
-update_titlebar_cb (gpointer call_obj,
-                    gpointer call_arg,
-                    gpointer user_data)
+static void
+update_titlebar_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 {
 	((ManagerUI*)(user_data))->titlebar_needs_refresh = TRUE;
-	return 0;
 }
 static void
 remove_update_titlebar_callbacks_cb (GtkObject * o, gpointer user_data)
