@@ -390,28 +390,26 @@ task_save_run (Task * task)
 
 	/**
 	*** Get the article bodies that we don't already have
+	*** (nntp_download_bodies will just skip over any
+	*** that are already in acache)
 	**/
 
 	if (1)
 	{
 		GSList * l;
 		gint index = 0;
-		GPtrArray * needbodies = g_ptr_array_new ();
-		for (l=articles; l!=NULL; l=l->next) {
-			Article * a = ARTICLE(l->data);
-			if (!acache_has_message(article_get_message_id(a)))
-				g_ptr_array_add (needbodies, a);
-		}
-		if (needbodies->len) {
+		GPtrArray * article_array = g_ptr_array_new ();
+		for (l=articles; l!=NULL; l=l->next)
+			g_ptr_array_add (article_array, l->data);
+		if (article_array->len)
 			result = nntp_download_bodies (STATUS_ITEM(task),
 						       TASK_SAVE(task)->group,
 						       task->sock,
 						       &task->hint_abort,
-						       needbodies,
+						       article_array,
 						       &index,
 						       FALSE, TRUE);
-		}
-		g_ptr_array_free (needbodies, TRUE);
+		g_ptr_array_free (article_array, TRUE);
 
 		if (result != TASK_SUCCESS) {
 			status_item_emit_error_va (STATUS_ITEM(task),
@@ -456,15 +454,26 @@ task_save_run (Task * task)
 		for (l=articles; l!=NULL; l=l->next)
 		{
 			const Article * a = ARTICLE(l->data);
-			gchar * txt = article_get_message (a);
-			gchar * fname;
-			gchar * full_fname;
+			char * txt = article_get_message (a);
+			char * fname;
+			char * full_fname;
+			char * path;
 			FILE * fp;
 
 			/* get the filename */
 			fname = g_strdup (TASK_SAVE(task)->filename_bodies);
 			if (!is_nonempty_string(fname))
-				replace_gstr (&fname, g_strdup_printf ("%s.txt", article_get_subject(a)));
+				replace_gstr (&fname, g_strdup_printf ("%s.txt", TASK_SAVE(task)->readable_name));
+
+			replace_gstr (&fname, get_unique_fname (TASK_SAVE(task)->path_bodies, fname));
+
+			/* get the directory */
+			path = g_dirname (fname);
+			if (!directory_check (path)) {
+				log_add_va (LOG_ERROR, _("Save bodies can't access path \"%s\""), g_dirname(fname));
+				bodies_ok = FALSE;
+			}
+			
 			full_fname = get_unique_fname (TASK_SAVE(task)->path_bodies, fname);
 
 			/* write the file */
@@ -485,6 +494,7 @@ task_save_run (Task * task)
 			/* cleanup */
 			g_free (txt);
 			g_free (fname);
+			g_free (path);
 			g_free (full_fname);
 		}
 
@@ -664,14 +674,14 @@ choose_article_parts_manual (TaskSave  * task,
 		Article * article = ARTICLE (l->data);
 		const struct tm local_tm = *localtime (&article->date);
 		gchar * text[4];
-		gchar * author = article_get_author_str (article);
+		gchar author[512];
 		int row;
 		sprintf (lines, "%d", article->linecount);
 		strftime (datebuf, sizeof(datebuf), "%Y/%m/%d %H:%M", &local_tm);
 
 	       	text[0] = (gchar*) article_get_subject (article);
 		text[1] = lines;
-		text[2] = author;
+		text[2] = article_get_author_str (article, author, sizeof(author));
 		text[3] = datebuf;
 
 		row = gtk_clist_append (GTK_CLIST(list), text);
@@ -681,8 +691,6 @@ choose_article_parts_manual (TaskSave  * task,
 			gtk_clist_select_row (GTK_CLIST(list), row, 0);
 			last_part = article->part;
 		}
-
-		g_free (author);
 	}
 	w = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(w),
