@@ -30,6 +30,7 @@
 #include <gmime/gmime.h>
 
 #include <pan/base/argset.h>
+#include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-object.h>
@@ -77,7 +78,6 @@ static gchar welcome[] =
 };
 
 static GMimeMessage * current_mm = NULL;
-static gboolean rot13_enabled = FALSE;
 static gboolean still_on_welcome = TRUE;
 static Article * current_article = NULL;
 static GtkWidget * header_table = NULL;
@@ -378,12 +378,12 @@ get_quote_color (const gchar * line)
 
 	if (is_nonempty_string(line))
 	{
-		gint depth = 0;
+		int depth = 0;
 
 		while (isspace((guchar)*line))
 			++line;
-		while (*line=='>' || isspace((guchar)*line)) {
-			if (*line=='>')
+		while (isspace((guchar)*line) || is_quote_char((guchar)*line)) {
+			if (is_quote_char((guchar)*line))
 				++depth;
 			++line;
 		}
@@ -592,7 +592,7 @@ text_set_raw (const gchar * text)
  * Used for displaying attached pictures inline.
  */
 static GtkWidget*
-get_gtk_pixmap_from_gmime_part (const GMimePart * part)
+get_gtk_pixmap_from_gmime_part (GMimePart * part)
 {
 	guint len;
 	const gchar * content;
@@ -601,10 +601,21 @@ get_gtk_pixmap_from_gmime_part (const GMimePart * part)
 	GdkBitmap * mask = NULL;
 	GtkWidget * w = NULL;
 	GdkPixbufLoader * l = NULL;
+	YencInfo * yenc;
 
 	/* create the loader */
 	l = gdk_pixbuf_loader_new ();
 
+	/* FIXME: this is a hack to get single-part yenc messages working.
+	   Ideally some pan-gmime mediator should have decoded this already */
+	yenc = GMIME_OBJECT(part)->user_data;
+	if (yenc!= NULL && part->content!=NULL && part->content->stream!=NULL) {
+		GMimeStream * stream = g_mime_stream_filter_new_with_stream (part->content->stream);
+		g_mime_stream_filter_add (GMIME_STREAM_FILTER(stream),
+		                          g_mime_filter_yenc_new (GMIME_FILTER_YENC_DIRECTION_DECODE));
+		g_mime_data_wrapper_set_stream (part->content, stream);
+	}
+
 	/* create the pixbuf */
 	content = g_mime_part_get_content (part, &len);
 	gdk_pixbuf_loader_write (l, (const guchar*)content, len);
@@ -654,7 +665,8 @@ extract_text_cb (GMimePart * part, gpointer data)
 	GString * body = (GString*) data;
 
 	const GMimeContentType * type = g_mime_part_get_content_type (part);
-	if (g_mime_content_type_is_type (type, "text", "plain"))
+	if (g_mime_content_type_is_type (type, "text", "plain")
+	 || g_mime_content_type_is_type (type, "text", "html"))
 	{
 		guint len;
 		const gchar * content;
@@ -667,8 +679,6 @@ extract_text_cb (GMimePart * part, gpointer data)
 
 		if (is_nonempty_string(pch))
 		{
-			if (rot13_enabled)
-				rot13_inplace (pch);
                 	if (text_get_wrap())
 				replace_gstr (&pch, fill_body(pch,wrap_column));      
 		
@@ -883,7 +893,7 @@ set_current_article_impl (Article * a, const char * text)
 	**/
 
 	if (text != NULL)
-		current_mm = pan_g_mime_parser_construct_message (text);
+		current_mm = pan_g_mime_parser_construct_message (&text, 1);
 
 	/**
 	***  Fire an event that the current article has changed
@@ -1134,19 +1144,34 @@ group_articles_removed_cb (gpointer call_obj, gpointer call_arg, gpointer client
 	debug_exit ("group_articles_removed_cb");
 }
 void
-text_set_rot13 (gboolean on)
+text_rot13_selected_text_nolock (void)
 {
-	debug_enter ("text_set_rot13");
+	GtkEditable * e;
+	gint a;
+	gint b;
+	gint start;
+	gint end;
 
-	if (on != rot13_enabled)
+	debug_enter ("text_rot13_selected_text_nolock");
+
+	e = GTK_EDITABLE(Pan.text);
+	if (e == NULL)
+		return;
+	a = e->selection_start_pos;
+	b = e->selection_end_pos;
+	start = MIN (a, b);
+	end = MAX (a, b);
+
+	if (end > start)
 	{
-		rot13_enabled = on;
-
-		if (current_article != NULL)
-			text_set_from_cached_article (current_article);
+		gchar * str = gtk_editable_get_chars (e, start, end);
+		rot13_inplace (str);
+		gtk_editable_delete_text (e, start, end);
+		gtk_editable_insert_text (e, str, strlen(str), &start);
+		gtk_editable_select_region (e, start, end);
 	}
-
-	debug_exit ("text_set_rot13");
+	
+	debug_exit ("text_rot13_selected_text_nolock");
 }
 
 /****
