@@ -24,7 +24,6 @@
 #include <gdk/gdk.h>
 #include <gtk/gtkwidget.h>
 #include <pan/base/pan-callback.h>
-#include <pan/message-window.h>
 
 /**
 ***  CURRENT ARTICLE
@@ -39,8 +38,6 @@ extern PanCallback * current_article_changed;
 
 extern Article * get_current_article (void);
 
-extern void set_current_article (Article*);
-
 /**
 ***  Wrapping
 **/
@@ -69,7 +66,7 @@ PanCallback*  text_get_mute_quoted_changed_callback          (void);
 ***  Rot 13
 **/
 
-void          text_set_rot13                                 (gboolean);
+void          text_rot13_selected_text_nolock                (void);
 
 /**
 ***  Colors
@@ -77,50 +74,32 @@ void          text_set_rot13                                 (gboolean);
 
 extern GdkColor text_fg_color;
 extern GdkColor text_bg_color;
-extern GdkColor text_quoted_color;
+extern GdkColor text_url_color;
+extern GdkColor text_quoted_color[3];
 
 
 /**
 ***
 **/
 
-void         text_read_more               (void);
-
-GtkWidget*   text_create                  (void);
-
-void         text_select_all              (void);
-
-void         text_refresh                 (void);
+GtkWidget*   text_create                     (void);
+void         text_read_more                  (void);
+void         text_select_all                 (void);
+void         text_refresh                    (void);
+void         text_clear_nolock               (void);
+void         text_set_from_cached_article    (Article      * article);
+void         text_set_from_mem_article       (Article      * article, const gchar * text);
+void         text_set_from_article           (Article      * article);
+void         text_set_raw                    (const gchar  * text);
 
 
-/**
- * This will display the message, if it exists in the acache.
- * If it's not in the acache, this method does nothing.
- * @param article the article to show
- */
-void      text_update                     (Article             * article);
+void         text_set_font                   (void);
 
-/**
- * This will add a task that tries to load the article's body 
- * from a local cache or from the news server.
- * If it's able to load the body, it will show it.
- */
-void      text_set_from_article           (Article             * article,
-                                           MessageWindowType     type,
-                                           gboolean              needs_window);
 
-void      text_set_raw                    (const gchar         * text);
+gchar*       text_get_message_to_reply_to    (void);
 
-gchar*    text_get_message_to_reply_to    (void);
-
-void      text_set_font                   (const gchar         * font_name);
-
-void      update_body_pane_nolock         (GtkWidget           * body_widget,
-	                                   const gchar         * body,
-	                                   gboolean              mute_quotes);
-
-void      update_body_pane                (GtkWidget           * body_widget,
-	                                   const gchar         * body,
-					   gboolean              mute_quotes);
+void         update_body_pane                (GtkWidget    * body_widget,
+	                                      const gchar  * body,
+					      gboolean     mute_quotes);
 
 #endif /* __TEXT_H__ */
